@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #  bln.py - a easyballoon compatible Saori module for ninix
-#  Copyright (C) 2002-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2011 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -30,6 +30,7 @@ if 'DISPLAY' in os.environ:
     import gtk
     import glib
     import pango
+    import cairo
     import ninix.pix
 else:
     gtk = None
@@ -152,7 +153,8 @@ class Balloon:
         self.name = name
         self.id = bln_id
         self.data = data # XXX
-        self.window = self.create_window()
+        self.window = ninix.pix.TransparentWindow(type=gtk.WINDOW_POPUP)
+        self.window.connect('delete_event', self.delete)
         self.position = data.get('position', 'sakura')
         left, top, scrn_w, scrn_h = ninix.pix.get_workarea()
         # -1: left, 1: right
@@ -187,7 +189,9 @@ class Balloon:
         h = max(8, int(h * self.scale / 100))
         balloon_pixbuf = balloon_pixbuf.scale_simple(
             w, h, gtk.gdk.INTERP_BILINEAR)
-        balloon_pixmap, mask_pixmap = balloon_pixbuf.render_pixmap_and_mask(1)
+        mask_pixmap = gtk.gdk.Pixmap(None, w, h, 1)
+        balloon_pixbuf.render_threshold_alpha(
+            mask_pixmap, 0, 0, 0, 0, w, h, 1)
         self.x, self.y = self.get_coordinate(w, h)
         if 'offset.x' in data:
             self.x += self.direction * int(int(data['offset.x']) * self.scale / 100)
@@ -204,28 +208,39 @@ class Balloon:
         self.right = int(int(data.get('disparea.right', w)) * self.scale / 100)
         self.top = int(int(data.get('disparea.top', 0)) * self.scale / 100)
         self.bottom = int(int(data.get('disparea.bottom', h)) * self.scale / 100)
-        self.window.shape_combine_mask(mask_pixmap, 0, 0)
+        if self.window.is_composited():
+            self.window.input_shape_combine_mask(mask_pixmap, 0, 0)
+        else:
+            self.window.shape_combine_mask(mask_pixmap, 0, 0)
+        self.current_mask = mask_pixmap
+        self.current_balloon_pixbuf = balloon_pixbuf
         self.script = None
         self.darea = gtk.DrawingArea()
-        self.darea.set_events(gtk.gdk.EXPOSURE_MASK)
+        self.darea.set_events(gtk.gdk.EXPOSURE_MASK|
+                              gtk.gdk.BUTTON_PRESS_MASK|
+                              gtk.gdk.BUTTON_RELEASE_MASK|
+                              gtk.gdk.POINTER_MOTION_MASK|
+                              gtk.gdk.LEAVE_NOTIFY_MASK)
         self.darea.connect('expose_event', self.redraw)
+        self.darea.connect('button_press_event', self.button_press)
+        self.darea.connect('button_release_event', self.button_release)
+        self.darea.connect('motion_notify_event', self.motion_notify)
+        self.darea.connect('leave_notify_event', self.leave_notify)
         self.darea.set_size_request(w, h)
         self.darea.show()
         self.window.add(self.darea)
         self.darea.realize()
-        self.surface_gc = self.darea.window.new_gc()
-        self.darea.window.set_back_pixmap(balloon_pixmap, False)
+        self.window.realize()
+        self.window.window.set_back_pixmap(None, False)
         self.layout = None
         if text != 'noscript' and \
            (self.right - self.left) and (self.bottom - self.top):
             self.script = text
             if 'font.color' in data:
-                fontcolor_r = int(data['font.color'][:2], 16)
-                fontcolor_g = int(data['font.color'][2:4], 16)
-                fontcolor_b = int(data['font.color'][4:6], 16)
-                cmap = self.darea.get_colormap()
-                self.surface_gc.foreground = cmap.alloc_color(
-                    '#%02x%02x%02x' % (fontcolor_r, fontcolor_g, fontcolor_b))
+                fontcolor_r = int(data['font.color'][:2], 16) / 255.0
+                fontcolor_g = int(data['font.color'][2:4], 16) /255.0
+                fontcolor_b = int(data['font.color'][4:6], 16) / 255.0
+                self.fontcolor = (fontcolor_r, fontcolor_g, fontcolor_b)
             default_font_size = 12 # for Windows environment
             font_size = int(data.get('font.size', default_font_size))
             font_size = font_size * 3 / 4 # convert from Windows to GTK+
@@ -312,19 +327,6 @@ class Balloon:
     def dragmove_vertical(self):
         return self.data.get('dragmove.vertical') == 'on'
 
-    def create_window(self):
-        window = gtk.Window(type=gtk.WINDOW_POPUP)
-        window.connect('delete_event', self.delete)
-        window.connect('button_press_event', self.button_press)
-        window.connect('button_release_event', self.button_release)
-        window.connect('motion_notify_event', self.motion_notify)
-        window.connect('leave_notify_event', self.leave_notify)
-        window.set_events(gtk.gdk.BUTTON_PRESS_MASK|
-                          gtk.gdk.BUTTON_RELEASE_MASK|
-                          gtk.gdk.POINTER_MOTION_MASK|
-                          gtk.gdk.LEAVE_NOTIFY_MASK)
-        return window
-
     def get_coordinate(self, w, h):
         left, top, scrn_w, scrn_h = ninix.pix.get_workarea()
         s0_x, s0_y, s0_w, s0_h = self.get_sakura_status('SurfaceSakura')
@@ -524,12 +526,20 @@ class Balloon:
         self.talking = 0 if self.talking and not sakura_talking else sakura_talking
         return True
 
-    def redraw(self, widget, event):
-        x, y, w, h = event.area
-        self.darea.window.clear()
+    def redraw(self, darea, event):
+        cr = darea.window.cairo_create()
+        cr.save()
+        cr.set_operator(cairo.OPERATOR_CLEAR)
+        cr.paint()
+        cr.restore()
+        cr.set_source_pixbuf(self.current_balloon_pixbuf, 0, 0)
+#        cr.paint_with_alpha(self.__alpha_channel)
+        cr.paint()
         if self.layout:
-            self.darea.window.draw_layout(self.surface_gc,
-                                          self.left, self.top, self.layout)
+            cr.set_source_rgb(*self.fontcolor)
+            cr.move_to(self.left, self.top)
+            cr.show_layout(self.layout)
+        del cr
 
     def get_state(self):
         return self.state
@@ -593,7 +603,7 @@ class Balloon:
         self.layout.set_text(text)
         self.darea.queue_draw()
 
-    def button_press(self, window, event):
+    def button_press(self, widget, event):
         self.x_root = event.x_root
         self.y_root = event.y_root
         if event.type == gtk.gdk._2BUTTON_PRESS:
@@ -603,7 +613,7 @@ class Balloon:
                                        x, y, self.id)
         return True
 
-    def button_release(self, window, event):
+    def button_release(self, widget, event):
         self.x_root = None
         self.y_root = None
         x = int(event.x * 100 / self.scale)
@@ -619,7 +629,7 @@ class Balloon:
             self.destroy()
         return True
 
-    def motion_notify(self, window, event):
+    def motion_notify(self, widget, event):
         if self.x_root is not None and \
            self.y_root is not None:
             x_delta = int(event.x_root - self.x_root)
@@ -642,7 +652,7 @@ class Balloon:
             self.move_notify_time = time.time()
         return True
 
-    def leave_notify(self, window, event):
+    def leave_notify(self, widget, event):
         self.move_notify_time = None
 
     def delete(self, window, event):
