@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #  hanayu.py - a "花柚" compatible Saori module for ninix
-#  Copyright (C) 2002-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2011 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -14,7 +14,7 @@
 
 # TODO:
 # - usetime, line, radar 以外の形式のグラフへの対応.
-# - タイトルの縦書き.(GTK+の対応待ち.)
+
 
 import sys
 import os
@@ -30,6 +30,7 @@ if 'DISPLAY' in os.environ:
             pass
     import gtk
     import pango
+    import cairo
     import ninix.pix
 else:
     gtk = None
@@ -47,10 +48,7 @@ class Saori(SAORI):
         self.data = {}
 
     def check_import(self):
-        if not gtk:
-            return 0
-        else:
-            return 1
+        return 0 if not gtk else 1
 
     def setup(self):
         self.dbpath = os.path.join(self.dir, self.__DBNAME)
@@ -239,6 +237,7 @@ class Saori(SAORI):
             return self.RESPONSE[400]
         return self.RESPONSE[204]
 
+
 class Graph:
 
     width = 450
@@ -251,9 +250,6 @@ class Graph:
         self.min = limit_min
         self.max = limit_max
         self.create_window()
-        self.draw_title()
-        self.draw_frame()
-        self.draw_graph()
         self.window.show()
 
     def create_window(self):
@@ -262,15 +258,15 @@ class Graph:
         self.window.set_decorated(False)
         self.window.set_resizable(False)
         self.window.connect('delete_event', self.delete)
-        self.window.connect('button_press_event', self.button_press)
-        self.window.set_events(gtk.gdk.BUTTON_PRESS_MASK)
         left, top, scrn_w, scrn_h = ninix.pix.get_workarea()
         self.x = left + scrn_w // 2
         self.y = top + scrn_h // 4
         self.window.move(self.x, self.y)
         self.darea = gtk.DrawingArea()
-        self.darea.set_events(gtk.gdk.EXPOSURE_MASK)
+        self.darea.set_events(gtk.gdk.EXPOSURE_MASK|
+                              gtk.gdk.BUTTON_PRESS_MASK)
         self.darea.connect('expose_event', self.redraw)
+        self.darea.connect('button_press_event', self.button_press)
         self.darea.set_size_request(self.width, self.height)
         self.darea.show()
         self.window.add(self.darea)
@@ -285,59 +281,55 @@ class Graph:
                 pixbuf = ninix.pix.create_pixbuf_from_file(path, is_pnr=0)
             except:
                 pixbuf = None
-        self.surface_pixmap = gtk.gdk.Pixmap(
-            None, self.width, self.height, self.darea.get_visual().depth)
-        cmap = self.darea.get_colormap()
-        self.surface_gc = self.surface_pixmap.new_gc()
-        self.surface_gc.foreground = cmap.alloc_color('#FFFFFF')
-        self.surface_pixmap.draw_rectangle(self.surface_gc, True, 0, 0,
-                                           self.width, self.height)
-        if pixbuf:
-            width = pixbuf.get_width()
-            height = pixbuf.get_height()
-            xoffset = (self.width - width) // 2
-            yoffset = (self.height - height) // 2
-            self.surface_pixmap.draw_pixbuf(self.surface_gc, pixbuf,
-                                            0, 0, xoffset, yoffset,
-                                            width, height,
-                                            gtk.gdk.RGB_DITHER_NONE, 0, 0)
-        self.darea.window.draw_drawable(self.surface_gc, self.surface_pixmap,
-                                        0, 0, 0, 0, self.width, self.height)
+        self.surface_pixbuf = pixbuf
+
+    def get_color(self, target):
+        assert target in ['font', 'line', 'frame', 'bar', 'background']
+        if target == 'background':
+            r = g = b = 255 # white
+        else:
+            r = g = b = 0 # black
+        name = ''.join((target, '.color'))
+        if name in self.data:
+            r = int(self.data[name][:2], 16)
+            g = int(self.data[name][2:4], 16)
+            b = int(self.data[name][4:6], 16)
+        else:
+            name_r = ''.join((name, '.r'))
+            if name_r in self.data:
+                r = int(self.data[name_r])
+            name_g = ''.join((name, '.g'))
+            if name_g in self.data:
+                g = int(self.data[name_g])
+            name_b = ''.join((name, '.b'))
+            if name_b in self.data:
+                b = int(self.data[name_b])
+        return (r / 255., g / 255., b / 255.)
 
     def draw_title(self):
-        fontcolor_r = fontcolor_g = fontcolor_b = 0
-        if 'font.color' in self.data:
-            fontcolor_r = int(self.data['font.color'][:2], 16)
-            fontcolor_g = int(self.data['font.color'][2:4], 16)
-            fontcolor_b = int(self.data['font.color'][4:6], 16)
-        else:
-            if 'font.color.r' in self.data:
-                fontcolor_r = int(self.data['font.color.r'])
-            if 'font.color.g' in self.data:
-                fontcolor_g = int(self.data['font.color.g'])
-            if 'font.color.b' in self.data:
-                fontcolor_b = int(self.data['font.color.b'])
-        cmap = self.darea.get_colormap()
-        self.surface_gc.foreground = cmap.alloc_color(
-            '#%02x%02x%02x' % (fontcolor_r, fontcolor_g, fontcolor_b))
         if 'title' in self.data:
             self.title = unicode(self.data['title'], 'Shift_JIS', 'ignore')
         font_size = 12 # pixel
         self.font_desc = pango.FontDescription()
         self.font_desc.set_family('Sans') # FIXME
         self.font_desc.set_size(font_size * pango.SCALE)
-        text = ''
-        i = 0
-        while i < len(self.title):
-            if text:
-                ''.join((text, '\n'))
-            ''.join((text, self.title[i]))
-            i += 1
-        self.layout.set_font_description(self.font_desc)
-        self.layout.set_wrap(pango.WRAP_CHAR)
-        self.layout.set_alignment(pango.ALIGN_CENTER)
-        self.layout.set_text(text)
-        self.surface_pixmap.draw_layout(self.surface_gc, 20, 58, self.layout)
+        cr = self.darea.window.cairo_create()
+        cr.set_source_rgb(*self.get_color('font'))
+        w, h = self.darea.window.get_size()
+        cr.translate(w, 0)
+        cr.rotate(math.pi / 2.0)
+        layout = cr.create_layout()
+        layout.set_font_description(self.font_desc)
+        context = layout.get_context()
+        default_gravity = context.get_gravity() # XXX
+        context.set_base_gravity(pango.GRAVITY_EAST) # Vertical Text
+        layout.set_text(self.title)
+        layout.set_wrap(pango.WRAP_WORD)
+        tw, th = layout.get_pixel_size()
+        cr.move_to(58, w - 20 - th)
+        cr.show_layout(layout)
+        context.set_base_gravity(default_gravity) # XXX
+        del cr
 
     def draw_frame(self):
         pass
@@ -345,11 +337,21 @@ class Graph:
     def draw_graph(self):
         pass
 
-    def redraw(self, widget, event):
-        x, y, w, h = event.area
-        if self.surface_pixmap:
-            self.darea.window.draw_drawable(
-                self.surface_gc, self.surface_pixmap, x, y, x, y, w, h)
+    def redraw(self, darea, event):
+        cr = darea.window.cairo_create()
+        cr.set_source_rgb(*self.get_color('background'))
+        cr.paint()
+        if self.surface_pixbuf:
+            width = self.surface_pixbuf.get_width()
+            height = self.surface_pixbuf.get_height()
+            xoffset = (self.width - width) // 2
+            yoffset = (self.height - height) // 2
+            cr.set_source_pixbuf(self.surface_pixbuf, xoffset, yoffset)
+            cr.paint()
+        del cr
+        self.draw_title()
+        self.draw_frame()
+        self.draw_graph()
 
     def button_press(self, window, event):
         if event.type == gtk.gdk.BUTTON_PRESS:
@@ -369,34 +371,25 @@ class Graph:
             self.window = None
             self.timeout_id = None
 
+
 class Line(Graph):
 
     def draw_frame(self):
-        framecolor_r = framecolor_g = framecolor_b = 0
-        if 'frame.color' in self.data:
-            framecolor_r = int(self.data['frame.color'][:2], 16)
-            framecolor_g = int(self.data['frame.color'][2:4], 16)
-            framecolor_b = int(self.data['frame.color'][4:6], 16)
-        else:
-            if 'frame.color.r' in self.data:
-                framecolor_r = int(self.data['frame.color.r'])
-            if 'frame.color.g' in self.data:
-                framecolor_g = int(self.data['frame.color.g'])
-            if 'frame.color.b' in self.data:
-                framecolor_b = int(self.data['frame.color.b'])
-        cmap = self.darea.get_colormap()
-        self.surface_gc.foreground = cmap.alloc_color(
-            '#%02x%02x%02x' % (framecolor_r, framecolor_g, framecolor_b))
         frame_width = 2
         if 'frame.width' in self.data:
             frame_width = int(self.data['frame.width'])
-        self.surface_gc.line_width = frame_width
-        self.surface_pixmap.draw_line(self.surface_gc,
-                                      60, 48, 60, 260)
-        self.surface_pixmap.draw_line(self.surface_gc,
-                                      60, 260, 420, 260)
+        cr = self.darea.window.cairo_create()
+        cr.set_line_width(frame_width)
+        cr.set_source_rgb(*self.get_color('frame'))
+        cr.move_to(60, 48)
+        cr.line_to(60, 260)
+        cr.line_to(420, 260)
+        cr.stroke()
+        del cr
 
     def draw_graph(self):
+        cr = self.darea.window.cairo_create()
+        cr.set_source_rgb(*self.get_color('font'))
         num = len(self.args) // 2
         step = 368 // num
         for index in range(num):
@@ -404,8 +397,8 @@ class Line(Graph):
             w, h = self.layout.get_pixel_size()
             pos_x = 60 + index * step + step // 2 - w // 2
             pos_y = 268
-            self.surface_pixmap.draw_layout(
-                self.surface_gc, pos_x, pos_y, self.layout)
+            cr.move_to(pos_x, pos_y)
+            cr.show_layout(self.layout)
         if self.min is not None:
             limit_min = self.min
         else:
@@ -420,93 +413,83 @@ class Line(Graph):
             for index in range(2, num):
                 if self.args[index * 2] > limit_max:
                     limit_max = self.args[index * 2]
-        linecolor_r = linecolor_g = linecolor_b = 0
-        if 'line.color' in self.data:
-            linecolor_r = int(self.data['line.color'][:2], 16)
-            linecolor_g = int(self.data['line.color'][2:4], 16)
-            linecolor_b = int(self.data['line.color'][4:6], 16)
-        else:
-            if 'line.color.r' in self.data:
-                linecolor_r = int(self.data['line.color.r'])
-            if 'line.color.g' in self.data:
-                linecolor_g = int(self.data['line.color.g'])
-            if 'line.color.b' in self.data:
-                linecolor_b = int(self.data['line.color.b'])
-        cmap = self.darea.get_colormap()
-        self.surface_gc.foreground = cmap.alloc_color(
-            '#%02x%02x%02x' % (linecolor_r, linecolor_g, linecolor_b))
         line_width = 2
         if 'line.width' in self.data:
             line_width = int(self.data['line.width'])
-        self.surface_gc.line_width = line_width
+        cr.set_line_width(line_width)
+        cr.set_source_rgb(*self.get_color('line'))
         for index in range(1, num):
             src_x = 60 + (index - 1) * step + step // 2
             src_y = 220 - int(
                 168 * self.args[(index - 1) * 2 + 1] / (limit_max - limit_min))
             dst_x = 60 + index * step + step // 2
             dst_y = 220 - int(168 * self.args[index * 2 + 1] / (limit_max - limit_min))
-            self.surface_pixmap.draw_line(self.surface_gc,
-                                          src_x, src_y, dst_x, dst_y)
+            cr.move_to(src_x, src_y)
+            cr.line_to(dst_x, dst_y)
+            cr.stroke()
         for index in range(num):
-            image, mask = None, None
+            pixbuf = None
             if self.args[index * 2 + 1] == limit_min and \
                'mark0.filename' in self.data:
                 path = os.path.join(
                     self.dir, self.data['mark0.filename'].replace('\\', '/'))
                 if os.path.exists(path):
                     try:
-                        image, mask = ninix.pix.create_pixmap_from_file(path)
+                        pixbuf = ninix.pix.create_pixbuf_from_file(path)
                     except:
-                        image, mask = None, None
+                        pixbuf = None
             elif self.args[index * 2 + 1] == limit_max and \
                  'mark2.filename' in self.data:
                 path = os.path.join(
                     self.dir, self.data['mark2.filename'].replace('\\', '/'))
                 if os.path.exists(path):
                     try:
-                        image, mask = ninix.pix.create_pixmap_from_file(path)
+                        pixbuf = ninix.pix.create_pixbuf_from_file(path)
                     except:
-                        image, mask = None, None
+                        pixbuf = None
             elif 'mark1.filename' in self.data:
                 path = os.path.join(
                     self.dir, self.data['mark1.filename'].replace('\\', '/'))
                 if os.path.exists(path):
                     try:
-                        image, mask = ninix.pix.create_pixmap_from_file(path)
+                        pixbuf = ninix.pix.create_pixbuf_from_file(path)
                     except:
-                        image, mask = None, None
-            if image:
-                w, h = image.get_size()
+                        pixbuf = None
+            if pixbuf:
+                w = pixbuf.get_width()
+                h = pixbuf.get_height()
                 x = 60 + index * step + step // 2 - w // 2
                 y = 220 - int(
                     168 * self.args[index * 2 + 1] / (limit_max - limit_min)) - h / 2
-                gc = self.surface_pixmap.new_gc()
-                gc.set_clip_mask(mask)
-                gc.set_clip_origin(x, y)
-                self.surface_pixmap.draw_drawable(gc, image, 0, 0, x, y, w, h)
+                cr.set_source_pixbuf(pixbuf, x, y)
+                cr.paint()
+        del cr
+
 
 class Bar(Graph):
 
-    def draw_graph(self):
-        barcolor_r = barcolor_g = barcolor_b = 0
-        if 'bar.color' in self.data:
-            barcolor_r = int(self.data['bar.color'][:2], 16)
-            barcolor_g = int(self.data['bar.color'][2:4], 16)
-            barcolor_b = int(self.data['bar.color'][4:6], 16)
-        ##else:
-        ##    if 'bar.color.r' in self.data:
-        ##        barcolor_r = int(self.data['bar.color.r'])
-        ##    if 'bar.color.g' in self.data:
-        ##        barcolor_g = int(self.data['bar.color.g'])
-        ##    if 'bar.color.b' in self.data:
-        ##        barcolor_b = int(self.data['bar.color.b'])
-        cmap = self.darea.get_colormap()
-        self.surface_gc.foreground = cmap.alloc_color(
-            '#%02x%02x%02x' % (barcolor_r, barcolor_g, barcolor_b))
+    def draw_frame(self):
+        frame_width = 2
+        if 'frame.width' in self.data:
+            frame_width = int(self.data['frame.width'])
+        cr = self.darea.window.cairo_create()
+        cr.set_line_width(frame_width)
+        cr.set_source_rgb(*self.get_color('frame'))
+        cr.move_to(60, 48)
+        cr.line_to(60, 260)
+        cr.line_to(420, 260)
+        cr.stroke()
+        del cr
+
+    def draw_graph(self): ## FIXME
+        cr = self.darea.window.cairo_create()
+        cr.set_source_rgb(*self.get_color('bar'))
+        bar_with = 20 ## FIXME
         if 'bar.width' in self.data:
-            pass # FIXME
+            bar_width = int(self.data['bar.width'])
         ### NOT YET ###
 
+
 class Radar(Graph):
 
     width = 288
@@ -516,31 +499,20 @@ class Radar(Graph):
         Graph.__init__(self, dir, data, args)
 
     def draw_frame(self):
-        framecolor_r = framecolor_g = framecolor_b = 0
-        if 'frame.color' in self.data:
-            framecolor_r = int(self.data['frame.color'][:2], 16)
-            framecolor_g = int(self.data['frame.color'][2:4], 16)
-            framecolor_b = int(self.data['frame.color'][4:6], 16)
-        else:
-            if 'frame.color.r' in self.data:
-                framecolor_r = int(self.data['frame.color.r'])
-            if 'frame.color.g' in self.data:
-                framecolor_g = int(self.data['frame.color.g'])
-            if 'frame.color.b' in self.data:
-                framecolor_b = int(self.data['frame.color.b'])
-        cmap = self.darea.get_colormap()
-        self.surface_gc.foreground = cmap.alloc_color(
-            '#%02x%02x%02x' % (framecolor_r, framecolor_g, framecolor_b))
         frame_width = 2
         if 'frame.width' in self.data:
             frame_width = int(self.data['frame.width'])
-        self.surface_gc.line_width = frame_width
+        cr = self.darea.window.cairo_create()
+        cr.set_line_width(frame_width)
+        cr.set_source_rgb(*self.get_color('frame'))
         num = len(self.args) // 2
         for index in range(num):
             x = 146 + int(math.cos(math.pi * (0.5 - 2.0 * index / num)) * 114)
             y = 146 - int(math.sin(math.pi * (0.5 - 2.0 * index / num)) * 114)
-            self.surface_pixmap.draw_line(self.surface_gc,
-                                          146, 146, x, y)
+            cr.move_to(146, 146,)
+            cr.line_to(x, y)
+            cr.stroke()
+        del cr
 
     def draw_graph(self):
         num = len(self.args) // 2
@@ -560,25 +532,12 @@ class Radar(Graph):
         for index in range(num):
             if self.args[index * 2 + 1] > limit_max:
                 limit_max = self.args[index * 2 + 1]
-        linecolor_r = linecolor_g = linecolor_b = 0
-        if 'line.color' in self.data:
-            linecolor_r = int(self.data['line.color'][:2], 16)
-            linecolor_g = int(self.data['line.color'][2:4], 16)
-            linecolor_b = int(self.data['line.color'][4:6], 16)
-        else:
-            if 'line.color.r' in self.data:
-                linecolor_r = int(self.data['line.color.r'])
-            if 'line.color.g' in self.data:
-                linecolor_g = int(self.data['line.color.g'])
-            if 'line.color.b' in self.data:
-                linecolor_b = int(self.data['line.color.b'])
-        cmap = self.darea.get_colormap()
-        self.surface_gc.foreground = cmap.alloc_color(
-            '#%02x%02x%02x' % (linecolor_r, linecolor_g, linecolor_b))
         line_width = 2
         if 'line.width' in self.data:
             line_width = int(self.data['line.width'])
-        self.surface_gc.line_width = line_width
+        cr = self.darea.window.cairo_create()
+        cr.set_line_width(line_width)
+        cr.set_source_rgb(*self.get_color('line'))
         if limit_max > 0:
             value = self.args[(num - 1) * 2 + 1] / limit_max
         else:
@@ -587,6 +546,7 @@ class Radar(Graph):
             math.pi * (0.5 - 2.0 * (num - 1) / num)) * value * 100)
         src_y = 146 - int(math.sin(
             math.pi * (0.5 - 2.0 * (num - 1) / num)) * value * 100)
+        cr.move_to(src_x, src_y)
         for index in range(num):
             if limit_max > 0:
                 value = self.args[index * 2 + 1] / limit_max
@@ -596,40 +556,29 @@ class Radar(Graph):
                 math.cos(math.pi * (0.5 - 2.0 * index / num)) * value * 100)
             dst_y = 146 - int(
                 math.sin(math.pi * (0.5 - 2.0 * index / num)) * value * 100)
-            self.surface_pixmap.draw_line(self.surface_gc,
-                                          src_x, src_y, dst_x, dst_y)
-            src_x = dst_x
-            src_y = dst_y
+            cr.line_to(dst_x, dst_y)
+        cr.stroke()
         font_size = 9 # pixel
         self.font_desc.set_size(font_size * pango.SCALE)
         self.layout.set_font_description(self.font_desc)
-        fontcolor_r = fontcolor_g = fontcolor_b = 0
-        if 'font.color' in self.data:
-            fontcolor_r = int(self.data['font.color'][:2], 16)
-            fontcolor_g = int(self.data['font.color'][2:4], 16)
-            fontcolor_b = int(self.data['font.color'][4:6], 16)
-        else:
-            if 'font.color.r' in self.data:
-                fontcolor_r = int(self.data['font.color.r'])
-            if 'font.color.g' in self.data:
-                fontcolor_g = int(self.data['font.color.g'])
-            if 'font.color.b' in self.data:
-                fontcolor_b = int(self.data['font.color.b'])
-        cmap = self.darea.get_colormap()
-        self.surface_gc.foreground = cmap.alloc_color(
-            '#%02x%02x%02x' % (fontcolor_r, fontcolor_g, fontcolor_b))
+        cr.set_source_rgb(*self.get_color('font'))
         for index in range(num):
-            if limit_max > 0:
-                value = self.args[index * 2 + 1] / limit_max
-            else:
-                value = 1.0
+            ##if limit_max > 0:
+            ##    value = self.args[index * 2 + 1] / limit_max
+            ##else:
+            ##    value = 1.0
+            value = 1.2 # XXX
             x = 146 + int(math.cos(
                 math.pi * (0.5 - 2.0 * index / num)) * value * 100)
             y = 146 - int(math.sin(
                 math.pi * (0.5 - 2.0 * index / num)) * value * 100)
             self.layout.set_text(self.args[index * 2])
-            self.surface_pixmap.draw_layout(
-                self.surface_gc, x, y + 12, self.layout)
+            w, h = self.layout.get_pixel_size()
+            x -= w // 2
+            y -= h // 2
+            cr.move_to(x, y)
+            cr.show_layout(self.layout)
+        del cr
 
 class Radar2(Graph):
 
