@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #  httpc.py - a HTTPC compatible Saori module for ninix
-#  Copyright (C) 2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2011 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -11,15 +11,18 @@
 #  PURPOSE.  See the GNU General Public License for more details.
 #
 
-import httplib
-import urlparse
 import glib
 
 try:
-    import chardet.universaldetector
+    import chardet
 except:
     chardet = None
 
+try:
+    import httplib2
+except:
+    httplib2 = None
+
 from ninix.dll import SAORI
 
 
@@ -29,50 +32,42 @@ class Saori(SAORI):
         SAORI.__init__(self)
         self.__sakura = None
         self.__bg = {}
+        self.conn = None
 
     def finalize(self):
         for timeout_id in self.__bg:
             glib.source_remove(timeout_id)
+        self.conn = None
+        return 1
 
     def need_ghost_backdoor(self, sakura):
         self.__sakura = sakura
 
+    def setup(self):
+        self.conn = httplib2.Http('.cache')
+        return 1
+
     def check_import(self):
-        return int(self.__sakura is not None and chardet is not None)
+        return int(self.__sakura is not None and \
+                       chardet is not None and \
+                       httplib2 is not None)
 
     def get(self, url, start=None, end=None):
-        url = urlparse.urlparse(url)
-        if not (url[0] == 'http' and url[3] == url[4] == url[5] == ''):
-            return self.RESPONSE[400]
-        conn = httplib.HTTPConnection(url[1])
-        conn.request("GET", url[2])
-        res = conn.getresponse()
-        if res.status != 200: # XXX
+        response, content = self.conn.request(url)
+        if response.status != 200: # XXX
             return []
-        data = res.read()
-        c_type = res.getheader('content-type')
+        c_type = response.get('content-type')
         if c_type:
             for x in c_type.split(';'):
                 if x.strip().startswith('charset='):
                     self.charset = x[9:]
                     break # XXX
-        else:
-            pass ## FIXME: check META http-equiv
         if self.charset is None:
             ## FIXME: respect explicit character encoding information
-            detector = chardet.universaldetector.UniversalDetector()
-            detector.reset()
-            for line in data:
-                detector.feed(line)
-                if detector.done:
-                    break
-            detector.close()
-            self.charset = detector.result['encoding']
-        if self.charset is not None:
-            data = unicode(data ,self.charset, 'ignore')
-        else:
-            pass ## FIXME
-        conn.close()
+            self.charset = chardet.detect(content)['encoding']
+        if self.charset is None:
+            return []
+        data = unicode(content, self.charset, 'ignore')
         if start is not None:
             assert end is not None
             nc = 0
@@ -145,11 +140,11 @@ class Saori(SAORI):
                 result = 'SAORI/1.0 200 OK\r\n' \
                     'Result: %s\r\n' % \
                     data[0].encode('Shift_JIS', 'replace')
-                for n in len(data):
-                    result = ''.join(
-                        result,
-                        'Value[%d]: %s\r\n' % \
-                            (n, data[n].encode('Shift_JIS', 'replace')))
+                for n in range(len(data)):
+                    result = ''.join((
+                            result,
+                            'Value%d: %s\r\n' % \
+                                (n, data[n].encode('Shift_JIS', 'replace'))))
                 result += '\r\n'
                 return result
                          
