@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #  mciaudior.py - a MCIAUDIOR compatible Saori module for ninix
-#  Copyright (C) 2003-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2003-2011 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2003 by MATSUMURA Namihiko <nie@counterghost.net>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -58,7 +58,7 @@ class Saori(SAORI):
                     self.state = 2
                     self.player.set_state(gst.STATE_PAUSED)
                     return self.RESPONSE[204]
-                if os.path.isfile(self.filepath):
+                if self.filepath is not None and os.path.isfile(self.filepath):
                     self.player.set_property("uri", "file://" + self.filepath)
                     self.player.set_state(gst.STATE_PLAYING)
         elif argc == 2:
