@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #  wmove.py - a wmove.dll compatible Saori module for ninix
-#  Copyright (C) 2003-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2003-2011 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -17,17 +17,7 @@
 import sys
 import os
 
-if 'DISPLAY' in os.environ:
-    if 'gtk' not in sys.modules:
-        try:
-            import pygtk
-            pygtk.require('2.0')
-        except ImportError:
-            pass
-    import gtk
-    import glib
-else:
-    gtk = None
+import glib
 
 from ninix.dll import SAORI
 import ninix.pix
@@ -47,7 +37,7 @@ class Saori(SAORI):
         self.timeout_id = None
         self.dir = dir
         result = 0
-        if not self.__sakura or not gtk:
+        if not self.__sakura:
             pass
         elif self.loaded:
 	    result = 2
