@@ -2,7 +2,7 @@
 #
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2011 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -21,7 +21,6 @@ import ninix.alias
 import ninix.dll
 
 NINIX_HOME = '~/.ninix'
-NINIX_USER = None
 
 def print_error(message):
     sys.stderr.write(''.join((message, '\n')))
@@ -29,28 +28,22 @@ def print_error(message):
 def get_ninix_home():
     return os.path.expanduser(NINIX_HOME)
 
-def get_ninix_user():
-    if NINIX_USER is None:
-        return get_ninix_home()
-    return os.path.expanduser(NINIX_USER)
-
 def get_pango_fontrc():
-    return os.path.join(get_ninix_user(), 'pango_fontrc')
+    return os.path.join(get_ninix_home(), 'pango_fontrc')
 
 def get_preferences():
-    return os.path.join(get_ninix_user(), 'preferences')
+    return os.path.join(get_ninix_home(), 'preferences')
 
 def load_config():
-    home_dir = get_ninix_home()
-    if not os.path.exists(home_dir):
+    if not os.path.exists(get_ninix_home()):
         return None
-    ghosts, shells = search_ghosts(home_dir)
-    balloons = search_balloons(home_dir)
-    plugins = search_plugins(home_dir)
-    nekoninni = search_nekoninni(home_dir)
-    katochan = search_katochan(home_dir)
-    kinoko = search_kinoko(home_dir)
-    return ghosts, shells, balloons, plugins, nekoninni, katochan, kinoko
+    ghosts = search_ghosts()
+    balloons = search_balloons()
+    plugins = search_plugins()
+    nekoninni = search_nekoninni()
+    katochan = search_katochan()
+    kinoko = search_kinoko()
+    return ghosts, balloons, plugins, nekoninni, katochan, kinoko
 
 def get_shiori(path):
     table = {}
@@ -68,9 +61,9 @@ def get_shiori(path):
                     table[name] = shiori
     return table
 
-def search_ghosts(home_dir, target=None):
+def search_ghosts(target=None):
+    home_dir = get_ninix_home()
     ghosts = []
-    shells = []
     if target:
         dirlist = []
         dirlist.extend(target)
@@ -97,11 +90,7 @@ def search_ghosts(home_dir, target=None):
                 candidate['name'] = name
                 candidate['score'] = score
         shell_name, surface_set = find_surface_set(prefix)
-        balloon = find_balloon(prefix)
         if candidate['score'] == 0:
-            if surface_set:
-                # use as a shell
-                shells.append((shell_name, surface_set, balloon))
             continue
         shiori_name = candidate['name']
         if desc.get('name') == 'default':
@@ -109,12 +98,14 @@ def search_ghosts(home_dir, target=None):
         else:
             pos = len(ghosts)
         use_makoto = find_makoto_dll(ghost_dir)
+        ## FIXME: check surface_set
         ghosts.insert(pos, (
-            desc, ghost_dir, use_makoto, surface_set, balloon, prefix,
+            desc, ghost_dir, use_makoto, surface_set, prefix,
             shiori_dll, shiori_name))
-    return ghosts, shells
+    return ghosts
 
-def search_balloons(home_dir):
+def search_balloons():
+    home_dir = get_ninix_home()
     balloon_dir = os.path.join(home_dir, 'balloon')
     try:
         dirlist = os.listdir(balloon_dir)
@@ -143,7 +134,8 @@ def search_balloons(home_dir):
         buf.insert(pos, (desc, balloon_info))
     return buf
 
-def search_plugins(home_dir):
+def search_plugins():
+    home_dir = get_ninix_home()
     buf = []
     plugin_dir = os.path.join(home_dir, 'plugin')
     try:
@@ -157,7 +149,8 @@ def search_plugins(home_dir):
         buf.append(plugin)
     return buf
 
-def search_nekoninni(home_dir): ## FIXME
+def search_nekoninni():
+    home_dir = get_ninix_home()
     buf = []
     skin_dir = os.path.join(home_dir, 'nekodorif/skin')
     try:
@@ -171,7 +164,8 @@ def search_nekoninni(home_dir): ## FIXME
         buf.append(nekoninni)
     return buf
 
-def search_katochan(home_dir): ## FIXME
+def search_katochan():
+    home_dir = get_ninix_home()
     buf = []
     katochan_dir = os.path.join(home_dir, 'nekodorif/katochan')
     try:
@@ -185,7 +179,8 @@ def search_katochan(home_dir): ## FIXME
         buf.append(katochan)
     return buf
 
-def search_kinoko(home_dir):
+def search_kinoko():
+    home_dir = get_ninix_home()
     buf = []
     kinoko_dir = os.path.join(home_dir, 'kinoko')
     try:
@@ -250,7 +245,7 @@ def read_kinoko_ini(top_dir):
     else:
         return None
 
-def read_profile_txt(top_dir): ## FIXME
+def read_profile_txt(top_dir):
     path = os.path.join(top_dir, 'profile.txt')
     name = None
     if is_readable(path):
@@ -264,7 +259,7 @@ def read_profile_txt(top_dir): ## FIXME
     else:
         return None
 
-def read_katochan_txt(top_dir): ## FIXME
+def read_katochan_txt(top_dir):
     path = os.path.join(top_dir, 'katochan.txt')
     katochan = {}
     katochan['dir'] = top_dir
@@ -549,18 +544,6 @@ def list_surface_elements(config):
         buf.append((key, method, filename, x, y))
     return buf
 
-def find_balloon(top_dir):
-    inst = read_install_txt(top_dir)
-    if inst:
-        balloon_dir = inst.get('balloon.directory')
-        if balloon_dir:
-            path = os.path.join(top_dir, 'ghost', 'master', balloon_dir.lower())
-            desc = read_descript_txt(path)
-            info = read_balloon_info(path)
-            if desc and info:
-                return (desc, info)
-    return None
-
 re_balloon = re.compile('balloon([skc][0-9]+)\.(png)')
 re_annex   = re.compile('(arrow[01]|sstp)\.(png)')
 
@@ -673,9 +656,9 @@ def test():
     config = load_config()
     if config is None:
         raise SystemExit, 'Home directory not found.\n'
-    ghosts, shells, balloons, plugins, nekoninni, katochan, kinoko = config ## FIXME
+    ghosts, balloons, plugins, nekoninni, katochan, kinoko = config
     # ghosts
-    for desc, shiori_dir, use_makoto, surface_set, balloon, prefix, shiori_dll, shiori_name in ghosts: ## FIXME
+    for desc, shiori_dir, use_makoto, surface_set, prefix, shiori_dll, shiori_name in ghosts:
         print 'GHOST', '=' * 50
         print prefix
         print str(desc).encode('utf-8', 'ignore')
@@ -707,46 +690,6 @@ def test():
                         for k, v in buf:
                             print k, '=', v
                         print
-        if balloon:
-            print '-' * 50
-            desc, balloon = balloon
-            print str(desc).encode('utf-8', 'ignore')
-            for k, v in balloon.iteritems():
-                print k, '=', v[0]
-                print str(v[1]).encode('utf-8', 'ignore')
-    # shells
-    for shell_name, surface_set, balloon in shells:
-        print 'SHELL', '=' * 50
-        print shell_name.encode('utf-8', 'ignore')
-        for name, surface_dir, desc, alias, surface in surface_set:
-            print '-' * 50
-            print 'surface:', name.encode('utf-8', 'ignore')
-            print str(desc).encode('utf-8', 'ignore')
-            for k, v in surface.iteritems():
-                print k, '=', v[0]
-                print str(v[1]).encode('utf-8', 'ignore')
-            if alias:
-                buf = []
-                for k, v in alias.iteritems():
-                    if k in ['sakura.surface.alias', 'kero.surface.alias']:
-                        print ''.join((k, ':'))
-                        for alias_id, alias_list in v.iteritems():
-                            print alias_id, ''.join(('= [', ', '.join(alias_list), ']'))
-                        print
-                    else:
-                        buf.append((k, v))
-                if buf:
-                    print 'filename alias:'
-                    for k, v in buf:
-                        print k, '=', v
-                    print
-        if balloon:
-            print '-' * 50
-            desc, balloon = balloon
-            print str(desc).encode('utf-8', 'ignore')
-            for k, v in balloon.iteritems():
-                print k, '=', v[0]
-                print str(v[1]).encode('utf-8', 'ignore')
     # balloons
     for desc, balloon in balloons:
         print 'BALLOON', '=' * 50
