@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-#  Copyright (C) 2004-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2004-2011 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -21,6 +21,7 @@ if 'DISPLAY' in os.environ:
     import gtk
     import glib
     import ninix.pix
+    import cairo
 
 import ninix.config
 import ninix.seriko
@@ -215,22 +216,16 @@ class Skin:
         self.__scale = scale
         self.__shown = 0
         self.surface_id = 0 # dummy
-        self.window = gtk.Window()
+        self.window = ninix.pix.TransparentWindow()
+        self.window.set_focus_on_map(False)
         ##self.window.set_title(''.join(('surface.', name)))
         self.window.set_decorated(False)
         self.window.set_resizable(False)
         self.window.set_skip_taskbar_hint(True)
         self.window.connect('delete_event', self.delete)
-        self.window.connect('button_press_event', self.button_press)
-        self.window.connect('button_release_event', self.button_release)
-        self.window.connect('motion_notify_event', self.motion_notify)
-        self.window.connect('leave_notify_event', self.leave_notify)
-        self.window.set_events(gtk.gdk.BUTTON_PRESS_MASK|
-                               gtk.gdk.BUTTON_RELEASE_MASK|
-                               gtk.gdk.POINTER_MOTION_MASK|
-                               gtk.gdk.LEAVE_NOTIFY_MASK)
         self.window.realize()
-        self.window.add_accel_group(self.accelgroup)
+        self.window.window.set_back_pixmap(None, False)
+        self.window.add_accel_group(self.accelgroup) ## FIXME
         if self.data['animation'] is not None:
             path = os.path.join(self.data['dir'], self.data['animation'])
             self.seriko = ninix.seriko.Controler(
@@ -250,22 +245,34 @@ class Skin:
             h = max(8, self.pixbuf.get_height() * self.__scale / 100)
             surface_pixbuf = self.pixbuf.scale_simple(
                 w, h, gtk.gdk.INTERP_BILINEAR)
-            image, mask = surface_pixbuf.render_pixmap_and_mask(255)
+            mask_pixmap = gtk.gdk.Pixmap(None, w, h, 1)
+            surface_pixbuf.render_threshold_alpha(
+                mask_pixmap, 0, 0, 0, 0, w, h, 1)
         except: ## FIXME
             self.callback['close'](None)
             return
-        ## FIXME
-        self.w, self.h = image.get_size()
+        self.w, self.h = w, h
+        self.current_surface_pixbuf = surface_pixbuf
+        self.current_mask = mask_pixmap
         self.set_position()
-        self.window.shape_combine_mask(mask, 0, 0)
+        if self.window.is_composited():
+            self.window.input_shape_combine_mask(mask_pixmap, 0, 0)
+        else:
+            self.window.shape_combine_mask(mask_pixmap, 0, 0)
         self.darea = gtk.DrawingArea()
-        self.darea.set_events(gtk.gdk.EXPOSURE_MASK)
+        self.darea.set_events(gtk.gdk.EXPOSURE_MASK|
+                               gtk.gdk.BUTTON_PRESS_MASK|
+                               gtk.gdk.BUTTON_RELEASE_MASK|
+                               gtk.gdk.POINTER_MOTION_MASK|
+                               gtk.gdk.LEAVE_NOTIFY_MASK)
+        self.darea.connect('button_press_event', self.button_press)
+        self.darea.connect('button_release_event', self.button_release)
+        self.darea.connect('motion_notify_event', self.motion_notify)
+        self.darea.connect('leave_notify_event', self.leave_notify)
         self.darea.connect('expose_event', self.redraw)
         self.darea.set_size_request(self.w, self.h)
         self.darea.show()
         self.window.add(self.darea)
-        self.darea.realize()
-        self.darea.window.set_back_pixmap(image, False)
         target_window = self.callback['get_target_window']()
         if self.data['ontop']:
             self.window.set_transient_for(target_window)
@@ -273,7 +280,7 @@ class Skin:
             target_window.set_transient_for(self.window)
         self.show()
         self.seriko.reset(self, '') # XXX
-        self.seriko.update(self)
+        self.seriko.start(self)
         self.seriko.invoke_kinoko(self)
 
     def show(self):
@@ -286,9 +293,6 @@ class Skin:
             self.window.hide()
             self.__shown = 0
 
-    def prefetch(self, surface_ids):
-        pass
-
     def append_actor(self, frame, actor):
         self.seriko.append_actor(frame, actor)
 
@@ -301,18 +305,25 @@ class Skin:
         offsety = self.data['offsety'] * self.__scale / 100
         self.x = base_x - int(self.w * a) + offsetx + xoffset
         self.y = base_y - int(self.h * b) + offsety + yoffset
-        self.window.move(self.x, self.y)
+        self.window.move(self.x, self.y) ## FIXME: same as surfaces
 
     def set_scale(self, scale):
         self.__scale = scale
-        self.update()
         self.set_position()
 
     def get_surface(self): ## FIXME
         return None
 
-    def redraw(self, widget, event): ## FIXME
-        pass
+    def redraw(self, darea, event):
+        cr = darea.window.cairo_create()
+        cr.save()
+        cr.set_operator(cairo.OPERATOR_CLEAR)
+        cr.paint()
+        cr.restore()
+        cr.set_source_pixbuf(self.current_surface_pixbuf, 0, 0)
+        ##cr.paint_with_alpha(self.__alpha_channel)
+        cr.paint()
+        del cr
 
     def get_pixbuf(self, surface_id):
         path = os.path.join(self.data['dir'], ''.join(('surface', str(surface_id), '.png')))
@@ -329,64 +340,52 @@ class Skin:
             surface_pixbuf = self.pixbuf.copy()
         return surface_pixbuf
 
-    def clear_frame_buffer(self):
-        self.frame_buffer = []
-
-    def update_frame_buffer(self, seriko, frame, move, dirty):
-        if dirty:
-            surface_pixbuf = self.create_surface_pixbuf(seriko.base_id)
-            if surface_pixbuf is not None:
-                # draw overlays
-                for pixbuf_id, x, y in seriko.iter_overlays():
-                    try:
-                        pixbuf = self.get_pixbuf(pixbuf_id)
-                        w = pixbuf.get_width()
-                        h = pixbuf.get_height()
-                    except:
-                        continue
-                    # overlay surface pixbuf
-                    sw = surface_pixbuf.get_width()
-                    sh = surface_pixbuf.get_height()
-                    if x + w > sw:
-                        w = sw - x
-                    if y + h > sh:
-                        h = sh - y
-                    if x < 0:
-                        dest_x = 0
-                        w += x
-                    else:
-                        dest_x = x
-                    if y < 0:
-                        dest_y = 0
-                        h += y
-                    else:
-                        dest_y = y
-                    pixbuf.composite(surface_pixbuf, dest_x, dest_y,
-                                     w, h, x, y, 1.0, 1.0,
-                                     gtk.gdk.INTERP_BILINEAR, 255)
-        else:
-            surface_pixbuf = None
-        self.frame_buffer.append((frame, move, surface_pixbuf, dirty))
-
-    def update(self):
-        if len(self.frame_buffer) > 0:
-            frame, move, surface_pixbuf, dirty = self.frame_buffer.pop(0)
-            if move is not None:
-                self.move_surface(*move)
-            if not dirty:
-                return
-        else:
-            surface_pixbuf = self.create_surface_pixbuf()
+    def update_frame_buffer(self):
+        surface_pixbuf = self.create_surface_pixbuf(self.seriko.base_id)
+        assert surface_pixbuf is not None
+        # draw overlays
+        for pixbuf_id, x, y in self.seriko.iter_overlays():
+            try:
+                pixbuf = self.get_pixbuf(pixbuf_id)
+                w = pixbuf.get_width()
+                h = pixbuf.get_height()
+            except:
+                continue
+            # overlay surface pixbuf
+            sw = surface_pixbuf.get_width()
+            sh = surface_pixbuf.get_height()
+            if x + w > sw:
+                w = sw - x
+            if y + h > sh:
+                h = sh - y
+            if x < 0:
+                dest_x = 0
+                w += x
+            else:
+                dest_x = x
+            if y < 0:
+                dest_y = 0
+                h += y
+            else:
+                dest_y = y
+            pixbuf.composite(surface_pixbuf, dest_x, dest_y,
+                             w, h, x, y, 1.0, 1.0,
+                             gtk.gdk.INTERP_BILINEAR, 255)
         w = max(8, surface_pixbuf.get_width() * self.__scale / 100)
         h = max(8, surface_pixbuf.get_height() * self.__scale / 100)
         surface_pixbuf = surface_pixbuf.scale_simple(
             w, h, gtk.gdk.INTERP_BILINEAR)
-        image, mask = surface_pixbuf.render_pixmap_and_mask(255)
-        self.w, self.h = image.get_size()
-        self.darea.window.set_back_pixmap(image, False)
-        self.window.shape_combine_mask(mask, 0, 0)
+        mask_pixmap = gtk.gdk.Pixmap(None, w, h, 1)
+        surface_pixbuf.render_threshold_alpha(
+            mask_pixmap, 0, 0, 0, 0, w, h, 1)
+        if self.window.is_composited():
+            self.window.input_shape_combine_mask(mask_pixmap, 0, 0)
+        else:
+            self.window.shape_combine_mask(mask_pixmap, 0, 0)
         #print 'DRAW:', dir(self.darea)
         #self.darea.queue_draw_area(0, 0, w, h)
+        self.current_mask = mask_pixmap
+        self.current_surface_pixbuf = surface_pixbuf
         self.darea.queue_draw()
 
     def terminate(self):
@@ -403,7 +402,6 @@ class Skin:
 
     def set_surface(self, surface_id, restart=1): ## FIXME
         self.pixbuf = self.get_pixbuf(surface_id)
-        self.update()
 
     def invoke(self, actor_id, update=0):
         self.seriko.invoke(self, actor_id, update)
@@ -411,8 +409,8 @@ class Skin:
     def delete(self, widget, event):
         self.callback['close'](None)
 
-    def destroy(self): ## FIXME
-        self.seriko.terminate(self)
+    def destroy(self):
+        self.seriko.destroy()
         self.window.destroy()
 
     def button_press(self, widget, event): ## FIXME
