@@ -3,7 +3,7 @@
 #
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2011 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2003-2005 by Shun-ichi TAHARA <jado@flowernet.gr.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -15,6 +15,8 @@
 #
 
 import optparse
+import urlparse
+import urllib
 import os
 import signal
 import socket
@@ -55,8 +57,6 @@ USAGE = 'Usage: ninix [options]'
 parser = optparse.OptionParser(USAGE)
 parser.add_option('-H', '--homedir', type='string', dest='homedir',
                   help='ninix home directory (default: ~/.ninix)')
-parser.add_option('-U', '--userdir', type='string', dest='userdir',
-                  help='user data directory (default: ninix home directory)')
 parser.add_option('--sstp-port', type='int', dest='sstp_port',
                   help='additional port for listening SSTP requests')
 parser.add_option('--debug', type='int', dest='debug', help='debug')
@@ -135,8 +135,6 @@ def main():
     # parse command line options
     home_dir = options.homedir if options.homedir is not None else \
         os.environ.get('NINIX_HOME')
-    user_dir = options.userdir if options.userdir is not None else \
-        os.environ.get('NINIX_USER')
     if options.sstp_port is not None:
         if options.sstp_port < 1024:
             sys.stderr.write('Invalid --sstp-port number (ignored)\n')
@@ -147,58 +145,22 @@ def main():
         if not os.path.isabs(os.path.expanduser(home_dir)):
             home_dir = os.path.join(os.getcwd(), home_dir)
         ninix.home.NINIX_HOME = home_dir
-    if user_dir:
-        if not os.path.isabs(os.path.expanduser(user_dir)):
-            user_dir = os.path.join(os.getcwd(), user_dir)
-        ninix.home.NINIX_USER = user_dir
-    config = ninix.home.load_config()
-    if config is None:
-        raise SystemExit, 'Home directory not found (abort)\n'
+    home_dir = ninix.home.get_ninix_home()
+    if not os.path.exists(home_dir):
+        try:
+            os.makedirs(home_dir)
+        except:
+            raise SystemExit, 'Cannot create Home directory (abort)\n'
     # aquire Inter Process Mutex (not Global Mutex)
     lockfile = open(os.path.join(ninix.home.get_ninix_home(), '.lock'), 'w')
     try:
         fcntl.flock(lockfile, fcntl.LOCK_EX | fcntl.LOCK_NB)
     except:
         raise SystemExit, 'ninix-aya is already running'
-    # check if at least one set of ghost, surface, and balloon
-    ghosts, shells, balloons, plugins = config[0:4]
-    if not ghosts:
-        raise SystemExit, 'Error: no ghost found\n'
-    elif not shells and \
-         not filter(lambda ghost: ghost[3], ghosts):
-        raise SystemExit, 'Error: no shell found\n'
-    elif not balloons:
-        raise SystemExit, 'Error: no balloon found\n'
-    for desc, surface_set, balloon in map(lambda g: (g[0], g[3], g[4]), ghosts):
-        print 'ghost:', desc.get('name', '<none>').encode('utf-8', 'ignore')
-        for name, surface_alias in map(lambda s: (s[0], s[3]), surface_set):
-            if name:
-                print '\tsurface:', name.encode('utf-8', 'ignore'),
-            if surface_alias is not None:
-                print '(w/alias)',
-            print
-        if balloon:
-            print '\tballoon:', \
-                  balloon[0].get('name', '<none>').encode('utf-8', 'ignore')
-    for shell_name, surface_set, balloon in shells:
-        print 'shell:', (shell_name or '<none>').encode('utf-8', 'ignore')
-        for name, surface_alias in map(lambda s: (s[0], s[3]), surface_set):
-            if name:
-                print '\tsurface:', name.encode('utf-8', 'ignore'),
-            if surface_alias is not None:
-                print '(w/alias)',
-            print
-        if balloon:
-            print '\tballoon:', \
-                  balloon[0].get('name', '<none>').encode('utf-8', 'ignore')
-    for desc in map(lambda b: b[0], balloons):
-        print 'balloon:', desc.get('name', '<none>').encode('utf-8', 'ignore')
-    for plugin_name in map(lambda p: p[0], plugins):
-        print 'plugin:', plugin_name.encode('utf-8', 'ignore')
     # start
     sys.stdout.write('loading...')
     sys.stdout.flush()
-    app = Application(config, sstp_port, debug)
+    app = Application(sstp_port, debug)
     sys.stdout.write('done.\n')
     app.run()
     try:
@@ -297,14 +259,15 @@ class SSTPControler:
 
 class Application(object):
 
-    def __init__(self, config, sstp_port=[9801, 11000], debug=0):
-        self.ghosts, self.shells, self.balloons, self.plugins, \
-                     self.nekoninni, self.katochan, self.kinoko = config ## FIXME
+    def __init__(self, sstp_port=[9801, 11000], debug=0):
+        self.loaded = False
+        self.confirmed = False
+        self.console = Console(self)
         self.debug = debug
         # create preference dialog
         prefs_callback = {
             'notify_preference_changed': self.notify_preference_changed,
-            'get_balloons': self.get_balloons,
+            'get_balloon_list': self.get_balloon_list,
             }
         self.prefs = ninix.prefs.PreferenceDialog(prefs_callback)
         sstp_controler_callback = {
@@ -323,8 +286,7 @@ class Application(object):
         plugin_callback = {
             'get_sstp_port': self.sstp_controler.get_sstp_port,
             }
-        self.plugin_controler = ninix.plugin.PluginControler(
-            self.plugins, plugin_callback)
+        self.plugin_controler = ninix.plugin.PluginControler(plugin_callback)
         # create usage dialog
         self.usage_dialog = UsageDialog()
         self.communicate = ninix.communicate.Communicate()
@@ -352,6 +314,7 @@ class Application(object):
             'close_ghost': self.close_ghost,
             'edit_preferences': self.prefs.edit_preferences,
             'find_balloon_by_name': self.find_balloon_by_name,
+            'find_balloon_by_subdir': self.find_balloon_by_subdir,
             'get_preference': self.prefs.get,
             'get_balloon_list': self.get_balloon_list,
             'get_balloon_description': self.get_balloon_description,
@@ -359,22 +322,19 @@ class Application(object):
             'get_kinoko_list': self.get_kinoko_list,
             'get_nekodorif_list': self.get_nekodorif_list,
             'get_plugin_list': self.get_plugin_list,
-            'get_shell_list': self.get_shell_list,
             'get_otherghostname': self.communicate.get_otherghostname,
             'open_ghost_manager': self.open_ghost_manager,
             'rebuild_ghostdb': self.communicate.rebuild_ghostdb,
             'reset_sstp_flag': self.sstp_controler.reset_sstp_flag,
-            'request_shell': self.request_shell,
-            'request_balloon': self.request_balloon,
             'show_usage': self.show_usage,
             'select_plugin': self.select_plugin,
             'select_kinoko': self.select_kinoko,
             'select_nekodorif': self.select_nekodorif,
             'select_ghost': self.select_ghost,
             'select_ghost_by_name': self.select_ghost_by_name,
-            'select_shell_by_name': self.select_shell_by_name,
             'start_sakura': self.start_sakura_cb,
             'stop_sakura': self.stop_sakura,
+            'reload_current_sakura': self.reload_current_sakura,
             'select_current_sakura': self.select_current_sakura,
             'send_message': self.communicate.send_message,
             'update_sakura': self.update_sakura,
@@ -427,7 +387,7 @@ class Application(object):
     def range_ghosts(self):
         return [i for i, sakura in enumerate(self.ghosts) if sakura is not None]
 
-    def __create_ghost_list_item(self, i): ## FIXME
+    def __create_ghost_list_item(self, i): ## FIXME: id
         if i >= len(self.ghosts):
             return None
         if self.ghosts[i] is None:
@@ -442,18 +402,6 @@ class Application(object):
                 icon_path = None
         else:
             icon_path = None
-        if icon_path is not None:
-            try:
-                pixbuf = ninix.pix.create_pixbuf_from_file(
-                    icon_path, is_pnr=False)
-            except:
-                pixbuf = None
-            if pixbuf is not None:
-                icon = pixbuf.scale_simple(16, 16, gtk.gdk.INTERP_BILINEAR)
-            else:
-                icon = None
-        else:
-            icon = None
         name = desc.get('name',
                         ''.join((unicode(_('Ghost'), 'utf-8'),
                                  '#%d' % (i + 1))))
@@ -465,7 +413,7 @@ class Application(object):
             shell_list.append((shell_name, value))
         item = {}
         item['name'] = name
-        item['icon'] = icon
+        item['icon'] = icon_path
         item['shell'] = shell_list
         item['instance'] = None
         return item
@@ -484,40 +432,13 @@ class Application(object):
             ghost_list.append(item)
         self.ghost_list = ghost_list
 
-    def get_shell_list(self): ## FIXME
-        set_type = 's'
-        set_list = []
-        for i in range(len(self.shells)):
-            name = self.shells[i][0]
-            if name is None:
-                name = unicode(''.join((_('Shell'), '#%d' % (i + 1))), 'utf-8')
-            shell_list = []
-            surface_set = self.shells[i][1]
-            for j in range(len(surface_set)):
-                value = (set_type, i, j)
-                shell_name = surface_set[j][0]
-                shell_list.append((shell_name, value))
-            item = {}
-            item['name'] = name
-            item['icon'] = None
-            item['shell'] = shell_list
-            set_list.append(item)
-        return set_list
-
-    def get_balloons(self):
-        return self.balloons
-
-    def get_balloon_list(self): ## FIXME
+    def get_balloon_list(self):
         balloon_list = []
-        for i in range(len(self.balloons)):
-            desc = self.balloons[i][0]
-            name = desc.get('name',
-                            ''.join((unicode(_('Balloon'), 'utf-8'),
-                                     '#%d' % (i + 1))))
-            item = {}
-            item['name'] = name
-            item['icon'] = None
-            balloon_list.append(item)
+        balloons = ninix.home.search_balloons()
+        for desc, balloon in balloons:
+            subdir = balloon['balloon_dir'][0]
+            name = desc.get('name', subdir)
+            balloon_list.append((name, subdir))
         return balloon_list
 
     def get_plugin_list(self): ## FIXME
@@ -525,7 +446,8 @@ class Application(object):
 
     def get_nekodorif_list(self):
         nekodorif_list = []
-        for nekoninni_name, nekoninni_dir in self.nekoninni:
+        nekoninni = ninix.home.search_nekoninni()
+        for nekoninni_name, nekoninni_dir in nekoninni:
             if not nekoninni_name:
                 continue
             item = {}
@@ -535,6 +457,7 @@ class Application(object):
         return nekodorif_list
 
     def get_kinoko_list(self):
+        self.kinoko = ninix.home.search_kinoko()
         return self.kinoko
 
     def load(self):
@@ -545,9 +468,8 @@ class Application(object):
         name = self.prefs.get('sakura_name')
         surface = self.prefs.get('sakura_surface', 0)
         default_sakura = self.find_ghost_by_name(name, surface) or \
-                         self.find_shell_by_name(name, surface) or \
                          self.choose_default_sakura()
-        # load ghost/shell
+        # load ghost
         self.current_sakura = default_sakura
         for i in range(len(self.ghosts)): ## FIXME
             sakura = self.create_ghost(self.ghosts[i], self.debug)
@@ -571,17 +493,6 @@ class Application(object):
                 pass
         return None
 
-    def find_shell_by_name(self, name, surface):
-        for i in range(len(self.shells)): ## FIXME
-            shell_name = self.shells[i][0]
-            try:
-                surface_set = self.shells[i][1]
-                if shell_name == name and surface < len(surface_set):
-                    return ('s', i, surface)
-            except: # old preferences(EUC-JP)
-                pass
-        return None
-
     def choose_default_sakura(self):
         for i in range(len(self.ghosts)): ## FIXME
             if self.ghosts[i] is None:
@@ -591,25 +502,37 @@ class Application(object):
         return ('s', 0, 0)
 
     def find_balloon_by_name(self, name):
-        for i in range(len(self.balloons)): ## FIXME
-            desc, balloon = self.balloons[i]
+        balloons = ninix.home.search_balloons()
+        for desc, balloon in balloons:
             try:
                 if desc.get('name') == name:
-                    return i
+                    return desc, balloon
                 if balloon['balloon_dir'][0] == name.encode('utf-8').lower(): # XXX
-                    return i
+                    return desc, balloon
+            except: # old preferences(EUC-JP)
+                pass
+        return None
+
+    def find_balloon_by_subdir(self, subdir):
+        balloons = ninix.home.search_balloons()
+        for desc, balloon in balloons:
+            try:
+                if balloon['balloon_dir'][0] == subdir:
+                    return desc, balloon
+                if desc.get('name').encode('utf-8').lower == subdir: # XXX
+                    return desc, balloon
             except: # old preferences(EUC-JP)
                 pass
         return None
 
     def run(self): ## FIXME
-        self.load()
-        # set SIGCHLD handler
-        signal.signal(signal.SIGCHLD, self.plugin_controler.terminate_plugin)
-        # start SSTP server
-        self.sstp_controler.start_servers()
-        # start plugins
-        self.plugin_controler.start_plugins()
+#        self.load()
+#        # set SIGCHLD handler
+#        signal.signal(signal.SIGCHLD, self.plugin_controler.terminate_plugin)
+#        # start SSTP server
+#        self.sstp_controler.start_servers()
+#        # start plugins
+#        self.plugin_controler.start_plugins()
         self.set_timeout()
         gtk.main()
 
@@ -624,8 +547,6 @@ class Application(object):
             name_list.append(name)
         return name_list
 
-### FIXME ###
-
     def get_ghost(self, if_ghost):
         for item in self.ghost_list:
             if item is None:
@@ -707,13 +628,11 @@ class Application(object):
         os.kill(0, signal.SIGHUP)
 
     def get_current_sakura_name(self): ## FIXME
-        # last ghost/shell
+        # last ghost
         set_type, i, j = self.current_sakura
-        if set_type == 'g':
-            desc = self.ghosts[i][0]
-            name = desc.get('name', '')
-        else:
-            name = self.shells[i][0]
+        assert set_type == 'g'
+        desc = self.ghosts[i][0]
+        name = desc.get('name', '')
         return name, str(j) # name, surface
 
     def save_preferences(self):
@@ -721,6 +640,8 @@ class Application(object):
             self.prefs.save()
         except IOError:
             sys.stderr.write('Cannot write preferences to file (ignored).\n')
+        except:
+            pass ## FIXME
 
     def select_ghost(self, sakura, sequential, event=1, vanished=0):
         if len(self.range_ghosts()) < 2:
@@ -741,23 +662,6 @@ class Application(object):
             self.current_sakura = ('g', i, 0)
         self.change_sakura(sakura, ('g', i, 0), 'automatic', event, vanished)
 
-    def select_shell_by_name(self, sakura, name, event=1):
-        set_type, i, j = sakura.current
-        assert len(self.ghosts) > i
-        desc, shiori_dir, use_makoto, surface_set, balloon, prefix, \
-            shiori_dll, shiori_name = self.ghosts[i]
-        if surface_set:
-            for j in range(len(surface_set)):
-                shell_name, surface_dir, surface_desc, surface_alias, surface, surface_tooltips = \
-                    surface_set[j]
-                if shell_name == name:
-                    break
-            else:
-                return
-        else:
-            return
-        self.change_sakura(sakura, (set_type, i, j), 'automatic', event)
-
     def select_ghost_by_name(self, sakura, name, event=1):
         item = self.find_ghost_by_name(name, 0)
         if item is None:
@@ -769,22 +673,13 @@ class Application(object):
         assert self.ghosts[i] is not None
         if sakura.current == item: # XXX: needs reloading?
             return
-        if set_type == 'g':
-            desc, shiori_dir, use_makoto, surface_set, balloon, prefix, \
-                  shiori_dll, shiori_name = self.ghosts[i]
-            if surface_set:
-                name, surface_dir, surface_desc, surface_alias, surface, surface_tooltips = \
-                      surface_set[j]
-            else:
-                shell_name, surface_set, balloon = self.shells[0]
-                name, surface_dir, surface_desc, surface_alias, surface, surface_tooltips = \
-                      surface_set[0]
-        elif set_type == 's':
-            desc, shiori_dir, use_makoto, surface_set, balloon, prefix, \
-                  shiori_dll, shiori_name = self.ghosts[0] ## FIXME: this should be default
-            shell_name, surface_set, shell_balloon = self.shells[i]
-            name, surface_dir, surface_desc, surface_alias, surface, surface_tooltips = \
-                  surface_set[j]
+        assert sakura.current[1] != i # XXX: not shell change
+        assert set_type == 'g'
+        desc, shiori_dir, use_makoto, surface_set, prefix, \
+            shiori_dll, shiori_name = self.ghosts[i]
+        assert surface_set and len(surface_set) > j
+        name, surface_dir, surface_desc, surface_alias, surface, surface_tooltips = \
+            surface_set[j]
         def proc(self=self, item=item):
             self.stop_sakura(sakura, self.start_sakura, item, sakura.current)
         if vanished:
@@ -793,9 +688,6 @@ class Application(object):
             self.close_ghost(sakura)
         elif not event:
             proc()
-        elif sakura.current[0] == set_type and sakura.current[1] == i:
-            assert sakura.current[2] != j
-            sakura.enqueue_event('OnShellChanging', name, surface_dir, proc=proc)
         else:
             name = surface_desc.get('sakura.name', desc.get('sakura.name'))
             sakura.enqueue_event('OnGhostChanging', name, method, proc=proc)
@@ -806,12 +698,6 @@ class Application(object):
             starter(*args)
         self.close_ghost(sakura)
 
-    def request_shell(self, index):
-        return self.shells[index]
-
-    def request_balloon(self, index):
-        return self.balloons[index]
-
     def select_ghost_from_list(self, item):
         set_type, i, j = item
         assert self.ghosts[i] is not None
@@ -846,29 +732,30 @@ class Application(object):
     def start_sakura_cb(self, event, item):
         self.start_sakura(item, init=1)
 
-    def get_balloon_description(self, name):
-        number = self.find_balloon_by_name(name)
-        if number is None:
+    def get_balloon_description(self, subdir):
+        balloon = self.find_balloon_by_subdir(subdir)
+        if balloon is None:
             ##print 'Balloon %s not found.' % name
             default_balloon = self.prefs.get('default_balloon')
-            number = self.find_balloon_by_name(default_balloon) or 0
-        desc, balloon = self.balloons[number]
-        return desc, balloon
+            balloon = self.find_balloon_by_subdir(default_balloon)
+        if balloon is None:
+            balloons = ninix.home.search_balloons()
+            assert balloons
+            balloon = balloons[0]
+        return balloon
 
     def reload_current_sakura(self, sakura):
         self.save_preferences()
         set_type, i, j = sakura.current
         if set_type == 's':
             i = 0
-        home_dir = ninix.home.get_ninix_home()
         ghost_dir = sakura.get_prefix()
-        ghost_conf, shell_conf = ninix.home.search_ghosts(home_dir,
-                                                          [ghost_dir])
+        ghost_conf = ninix.home.search_ghosts([ghost_dir])
         if ghost_conf:
             self.ghosts[i] = ghost_conf[0]
+            sakura.new(*ghost_conf[0]) # reset
         else:
-            self.ghosts[i] = None
-            self.shells.append(shell_conf[0]) ## FIXME
+            self.ghosts[i] = None ## FIXME
         self.update_ghost_list(i, sakura)
         item = (set_type, i, j)
         self.start_sakura(item, item, init=1) 
@@ -897,7 +784,8 @@ class Application(object):
         self.plugin_controler.select_plugin(event, item)
 
     def select_nekodorif(self, nekodorif_dir, target):
-        ninix.nekodorif.Nekoninni().load(nekodorif_dir, self.katochan, target)
+        ninix.nekodorif.Nekoninni().load(nekodorif_dir,
+                                         ninix.home.search_katochan(), target)
 
     def select_kinoko(self, data, target):
         ninix.kinoko.Kinoko(self.kinoko).load(data, target)
@@ -947,12 +835,134 @@ class Application(object):
     def set_timeout(self):
         self.timeout_id = glib.timeout_add(100, self.do_idle_tasks) # 100ms
 
+    def search_ghosts(self): ## FIXME
+        self.ghosts = ninix.home.search_ghosts()
+        balloons = self.get_balloon_list()
+        if len(self.ghosts) > 0 and len(balloons) > 0:
+            self.confirmed = True
+        return len(self.ghosts), len(balloons)
+
     def do_idle_tasks(self):
-        self.sstp_controler.handle_sstp_queue()
-        self.sstp_controler.receive_sstp_request()
+        if not self.confirmed:
+            self.console.open(exception=True)
+        else:
+            if not self.loaded:
+                self.load()
+                # set SIGCHLD handler
+                signal.signal(signal.SIGCHLD, self.plugin_controler.terminate_plugin)
+                # start SSTP server
+                self.sstp_controler.start_servers()
+                # start plugins
+                self.plugin_controler.start_plugins()
+                self.loaded = True
+            else:
+                self.sstp_controler.handle_sstp_queue()
+                self.sstp_controler.receive_sstp_request()
         self.set_timeout()
 
 
+class Console:
+
+    # DnD data types
+    dnd_targets = [
+        ('text/plain', 0, 0),
+        ]
+
+    def __init__(self, app):
+        self.app = app
+        self.window = gtk.Dialog()
+        self.window.connect('delete_event', self.close)
+        self.darea = gtk.DrawingArea()
+        self.darea.set_events(gtk.gdk.EXPOSURE_MASK)
+        self.darea.connect('drag_data_received', self.drag_data_received)
+        self.darea.drag_dest_set(gtk.DEST_DEFAULT_ALL, self.dnd_targets,
+                                 gtk.gdk.ACTION_COPY)
+        self.size = (330, 110) ## FIXME
+        self.darea.set_size_request(*self.size)
+        self.darea.connect('configure_event', self.configure)
+        self.darea.connect('expose_event', self.redraw)
+        self.window.vbox.pack_start(self.darea)
+        self.darea.show()
+        box = gtk.HButtonBox()
+        box.set_layout(gtk.BUTTONBOX_END)
+        self.window.action_area.pack_start(box)
+        box.show()
+        button = gtk.Button('Close')
+        button.connect('clicked', self.close)
+        box.add(button)
+        button.show()
+        self.opened = 0
+
+    def open(self, exception=False): ## FIXME
+        if self.opened:
+            return
+        self.window.show()
+        self.opened = 1
+
+    def close(self, widget=None, event=None):
+        self.window.hide()
+        self.opened = 0
+        if not self.app.confirmed: ## FIXME
+            self.app.quit()
+        return True
+
+    def configure(self, darea, event):
+        x, y, w, h = darea.get_allocation()
+        self.size = (w, h)
+
+    def draw_message(self, text): ## FIXME
+        pass
+
+    def redraw(self, darea, event):
+        ghosts, balloons = self.app.search_ghosts() # XXX
+        if ghosts > 0 and balloons > 0:
+            self.window.set_title(_('Console'))
+        else:
+            self.window.set_title(_('Nanntokashitekudasai.'))
+        layout = pango.Layout(darea.get_pango_context())
+        font_desc = pango.FontDescription()
+        font_desc.set_size(9 * pango.SCALE)
+        font_desc.set_family('Sans') # FIXME
+        layout.set_font_description(font_desc)
+        cr = darea.window.cairo_create()
+        w, h = self.size
+        cr.set_source_rgb(0.0, 0.0, 0.0) # black
+        cr.paint()
+        layout.set_text('Ghosts: %d' % ghosts)
+        if ghosts == 0:
+            cr.set_source_rgb(1.0, 0.2, 0.2) # red
+        else:
+            cr.set_source_rgb(0.8, 0.8, 0.8) # gray
+        cr.move_to(20, 15)
+        cr.show_layout(layout)
+        w, h = layout.get_pixel_size()
+        layout.set_text('Balloons: %d' % balloons)
+        if balloons == 0:
+            cr.set_source_rgb(1.0, 0.2, 0.2) # red
+        else:
+            cr.set_source_rgb(0.8, 0.8, 0.8) # gray
+        cr.move_to(20, 15 + h)
+        cr.show_layout(layout)
+        del cr
+
+    def drag_data_received(self, widget, context, x, y, data, info, time):
+        print 'Content-type:', data.type
+        print 'Content-length:', data.get_length()
+        ##print repr(data.data)
+        if str(data.type) == 'text/plain':
+            filelist = []
+            for line in data.data.split('\r\n'):
+                scheme, host, path, params, query, fragment = \
+                        urlparse.urlparse(line)
+                pathname = urllib.url2pathname(path)
+                if scheme == 'file' and os.path.exists(pathname):
+                    filelist.append(pathname)
+            for filename in filelist:
+                os.system('ninix-install %s' % filename) ## FIXME
+        self.darea.queue_draw()
+        return True
+
+
 class UsageDialog:
 
     def __init__(self):
@@ -1015,23 +1025,20 @@ class UsageDialog:
         total = float(0)
         for name, clock, path in self.items:
             total += clock
-        # prepare gc
         layout = pango.Layout(darea.get_pango_context())
         font_desc = pango.FontDescription()
         font_desc.set_size(9 * pango.SCALE)
         font_desc.set_family('Sans') # FIXME
         layout.set_font_description(font_desc)
-        white_gc = darea.get_style().white_gc
-        black_gc = darea.get_style().black_gc
-        cmap = darea.get_colormap()
-        gray_gc = darea.window.new_gc()
-        gray_gc.foreground = cmap.alloc_color('#cfcfcf')
+        cr = darea.window.cairo_create()
         # redraw graph
         w, h = self.size
-        darea.window.draw_rectangle(white_gc, True, 0, 0, w, h)
+        cr.set_source_rgb(1.0, 1.0, 1.0) # white
+        cr.paint()
         # ai.png
         if self.pixbuf:
-            darea.window.draw_pixbuf(None, self.pixbuf, 0, 0, 16, 32, -1, -1)
+            cr.set_source_pixbuf(self.pixbuf, 16, 32) # XXX
+            cr.paint()
         w3 = w4 = 0
         rows = []
         for name, clock, path in self.items[:14]:
@@ -1055,26 +1062,38 @@ class UsageDialog:
         label = 'name'
         layout.set_text(label)
         label_name_w, label_name_h = layout.get_pixel_size()
-        darea.window.draw_layout(gray_gc, x, y, layout)
+        cr.set_source_rgb(0.8, 0.8, 0.8) # gray
+        cr.move_to(x, y)
+        cr.show_layout(layout)
         x = x + w2 + 10
         label = 'rate'
         layout.set_text(label)
         label_rate_w, label_rate_h = layout.get_pixel_size()
-        darea.window.draw_layout(gray_gc, x + w3 - label_rate_w, y, layout)
+        cr.set_source_rgb(0.8, 0.8, 0.8) # gray
+        cr.move_to(x + w3 - label_rate_w, y)
+        cr.show_layout(layout)
         x += w3 + 10
         label = 'time'
         layout.set_text(label)
         label_time_w, label_time_h = layout.get_pixel_size()
-        darea.window.draw_layout(gray_gc, x + w4 - label_time_w, y, layout)
+        cr.set_source_rgb(0.8, 0.8, 0.8) # gray
+        cr.move_to(x + w4 - label_time_w, y)
+        cr.show_layout(layout)
         y += max([label_name_h, label_rate_h, label_time_h]) + 4
         for clock, name, name_w, name_h, rate, rate_w, rate_h, time, time_w, \
                 time_h  in rows:
             x = 20
             bw = int(clock / total * w1)
             bh = max([name_h, rate_h, time_h]) - 1
-            darea.window.draw_rectangle(gray_gc,  False, x + 1, y + 1, bw, bh)
-            darea.window.draw_rectangle(white_gc, False, x, y, bw, bh)
-            darea.window.draw_rectangle(black_gc, False, x, y, bw, bh)
+            cr.set_source_rgb(0.8, 0.8, 0.8) # gray
+            cr.rectangle(x + 1, y + 1, bw, bh)
+            cr.stroke()
+            cr.set_source_rgb(1.0, 1.0, 1.0) # white
+            cr.rectangle(x, y, bw, bh)
+            cr.stroke()
+            cr.set_source_rgb(0.0, 0.0, 0.0) # black
+            cr.rectangle(x, y, bw, bh)
+            cr.stroke()
             x += w1 + 10
             layout.set_text(name)
             end = len(name)
@@ -1085,14 +1104,21 @@ class UsageDialog:
                     layout.set_text(''.join((name[:end], u'...')))
                 else:
                     break
-            darea.window.draw_layout(black_gc, x, y, layout)
+            cr.set_source_rgb(0.0, 0.0, 0.0) # black
+            cr.move_to(x, y)
+            cr.show_layout(layout)
             x += w2 + 10
             layout.set_text(rate)
-            darea.window.draw_layout(black_gc, x + w3 - rate_w, y, layout)
+            cr.set_source_rgb(0.0, 0.0, 0.0) # black
+            cr.move_to(x + w3 - rate_w, y)
+            cr.show_layout(layout)
             x += w3 + 10
             layout.set_text(time)
-            darea.window.draw_layout(black_gc, x + w4 - time_w, y, layout)
+            cr.set_source_rgb(0.0, 0.0, 0.0) # black
+            cr.move_to(x + w4 - time_w, y)
+            cr.show_layout(layout)
             y += max([name_h, rate_h, time_h]) + 4
+        del cr
 
 
 if __name__ == '__main__':
