@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-#  Copyright (C) 2003-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2003-2011 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2003 by Shun-ichi TAHARA <jado@flowernet.gr.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -547,7 +547,11 @@ class Menu:
             self.__set_visible('Recommend', 0)
         self.__menu_list['Recommend'][3] = 1 # changed
 
-    def __update_ghost_menu(self, set_list, callback, summon):
+    def __update_ghost_menu(self, set_list, summon):
+        if summon:
+            callback = self.callback['start_sakura']
+        else:
+            callback = self.callback['select_sakura']
         ghost_menu = gtk.Menu()
         for i in range(len(set_list)):
             if set_list[i] is None:
@@ -557,7 +561,7 @@ class Menu:
             shell_list = set_list[i]['shell']
             if icon is not None:
                 image = gtk.Image()
-                image.set_from_pixbuf(icon)
+                image.set_from_file(icon)
                 image.show()
                 item = gtk.ImageMenuItem(name)
                 item.set_image(image)
@@ -566,11 +570,11 @@ class Menu:
             item.set_name('popup menu item')
             item.show()
             ghost_menu.append(item)
+            if set_list[i]['instance'].is_running():
+                item.set_sensitive(False)
             if len(shell_list) <= 1:
                 shell_name, value = shell_list[0]
                 item.connect('activate', callback, value)
-                if set_list[i]['instance'].is_running():
-                    item.set_sensitive(False)
             else:
                 submenu = gtk.Menu()
                 submenu.set_name('popup menu')
@@ -580,15 +584,6 @@ class Menu:
                     item.set_name('popup menu item')
                     item.connect('activate', callback, value)
                     item.show()
-                    if set_list[i]['instance'].is_running():
-                        if summon:
-                            item.set_sensitive(False)
-                        else:
-                            set_type, i, j = self.callback['get_current_item']()
-                            if i != value[1]:
-                                item.set_sensitive(False)
-                            elif j == value[2]:
-                                item.set_sensitive(False)
                     submenu.append(item)
         path = 'Summon' if summon else 'Change'
         menuitem = self.ui_manager.get_widget(''.join(('/popup/', path)))
@@ -597,37 +592,21 @@ class Menu:
 
     def __set_ghost_menu(self):
         ghosts = self.callback['get_ghost_list']()
-        self.__update_ghost_menu(ghosts, self.callback['select_sakura'], False)
-        self.__update_ghost_menu(ghosts, self.callback['start_sakura'], True)
+        self.__update_ghost_menu(ghosts, False)
+        self.__update_ghost_menu(ghosts, True)
 
     def __set_shell_menu(self):
-        set_list = self.callback['get_shell_list']()
+        shell_list = self.callback['get_shell_list']()
         shell_menu = gtk.Menu()
-        for i in range(len(set_list)):
-            name = set_list[i]['name']
-            item = gtk.MenuItem(name)
+        for i in range(len(shell_list)):
+            shell_name, value, working = shell_list[i]
+            item = gtk.MenuItem(shell_name)
             item.set_name('popup menu item')
+            item.connect('activate', self.callback['select_shell'], value)
             item.show()
+            if working:
+                item.set_sensitive(False)
             shell_menu.append(item)
-            shell_list = set_list[i]['shell']
-            if len(shell_list) <= 1:
-                shell_name, value = shell_list[0]
-                item.connect('activate', self.callback['select_sakura'], value)
-                ##if working:
-                ##    item.set_sensitive(False)
-            else:
-                submenu = gtk.Menu()
-                submenu.set_name('popup menu')
-                item.set_submenu(submenu)
-                for shell_name, value in shell_list:
-                    item = gtk.MenuItem(shell_name)
-                    item.set_name('popup menu item')
-                    item.connect('activate', self.callback['select_sakura'],
-                                 value)
-                    item.show()
-                    ##if working: ## FIXME
-                    ##    item.set_sensitive(False)
-                    submenu.append(item)
         menuitem = self.ui_manager.get_widget(''.join(('/popup/', 'Shell')))
         menuitem.set_submenu(shell_menu)
         self.__menu_list['Shell'][3] = 1
@@ -636,26 +615,14 @@ class Menu:
         balloon_list = self.callback['get_balloon_list']()
         balloon_menu = gtk.Menu()
         group = None
-        name_own = self.callback['get_own_balloon_name']()
-        if name_own is not None:
-            item = group = gtk.RadioMenuItem(group, name_own)
-            item.set_name('popup menu item')
-            hid = item.connect('activate', self.callback['select_balloon'],
-                               name_own)
-            item.show()
-            balloon_menu.append(item)
-        item = gtk.SeparatorMenuItem()
-        balloon_menu.add(item)
-        item.show()
-        for i in range(len(balloon_list)):
-            name = balloon_list[i]['name']
+        for name, directory in balloon_list:
             item = group = gtk.RadioMenuItem(group, name)
             item.set_name('popup menu item')
             hid = item.connect('activate',
-                               self.callback['select_balloon'], name)
+                               self.callback['select_balloon'], directory)
             item.show()
             balloon_menu.append(item)
-            if name == self.callback['get_current_balloon_name']():
+            if directory == self.callback['get_current_balloon_directory']():
                 item.handler_block(hid)
                 item.activate()
                 item.handler_unblock(hid)
