@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-#  Copyright (C) 2004-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2004-2011 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -52,6 +52,7 @@ import random
 if 'DISPLAY' in os.environ:
     import gtk
     import glib
+    import cairo
     import ninix.pix
 
 import ninix.home
@@ -117,6 +118,7 @@ class Menu:
         else:
             self.__menu_list[key][1].hide()
 
+
 class Nekoninni:
 
     def __init__(self):
@@ -255,28 +257,30 @@ class Skin:
         self.__menu = Menu(menu_callback, self.accelgroup)
         path = os.path.join(self.dir, 'omni.txt')
         self.omni = int(os.path.isfile(path) and os.path.getsize(path) == 0)
-        self.window = gtk.Window()
+        self.window = ninix.pix.TransparentWindow()
         name, top_dir = ninix.home.read_profile_txt(dir) # XXX
         self.window.set_title(name)
         self.window.set_decorated(False)
         self.window.set_resizable(False)
+        self.window.set_focus_on_map(False)
         self.window.connect('delete_event', self.delete)
-        self.window.connect('button_press_event', self.button_press)
-        self.window.connect('button_release_event', self.button_release)
-        self.window.connect('motion_notify_event', self.motion_notify)
-        self.window.connect('leave_notify_event', self.leave_notify)
-        self.window.set_events(gtk.gdk.BUTTON_PRESS_MASK|
-                               gtk.gdk.BUTTON_RELEASE_MASK|
-                               gtk.gdk.POINTER_MOTION_MASK|
-                               gtk.gdk.POINTER_MOTION_HINT_MASK|
-                               gtk.gdk.LEAVE_NOTIFY_MASK)
         self.window.realize()
+        self.window.window.set_back_pixmap(None, False)
         self.window.add_accel_group(self.accelgroup)
         self.darea = gtk.DrawingArea()
-        self.darea.set_events(gtk.gdk.EXPOSURE_MASK)
+        self.darea.set_events(gtk.gdk.EXPOSURE_MASK|
+                              gtk.gdk.BUTTON_PRESS_MASK|
+                              gtk.gdk.BUTTON_RELEASE_MASK|
+                              gtk.gdk.POINTER_MOTION_MASK|
+                              gtk.gdk.POINTER_MOTION_HINT_MASK|
+                              gtk.gdk.LEAVE_NOTIFY_MASK)
         self.darea.connect('expose_event', self.redraw)
+        self.darea.connect('button_press_event', self.button_press)
+        self.darea.connect('button_release_event', self.button_release)
+        self.darea.connect('motion_notify_event', self.motion_notify)
+        self.darea.connect('leave_notify_event', self.leave_notify)
         self.id = [0, None]
-        self.darea.show()
+        self.darea.show() ## FIXME
         self.window.add(self.darea)
         self.darea.realize()
         self.set_surface()
@@ -288,8 +292,16 @@ class Skin:
         self.set_surface()
         self.set_position()
 
-    def redraw(self, widget, event):
-        pass
+    def redraw(self, darea, event):
+        cr = darea.window.cairo_create()
+        cr.save()
+        cr.set_operator(cairo.OPERATOR_CLEAR)
+        cr.paint()
+        cr.restore()
+        cr.set_source_pixbuf(self.current_surface_pixbuf, 0, 0)
+        ##cr.paint_with_alpha(self.__alpha_channel)
+        cr.paint()
+        del cr
 
     def delete(self, widget, event):
         self.callback['finalize']()
@@ -326,18 +338,26 @@ class Skin:
         else:
             path = os.path.join(self.dir, 'surface%d.png' % self.id[0])
         try:
-            pixbuf = ninix.pix.create_pixbuf_from_file(path)
-            w = max(8, pixbuf.get_width() * self.__scale / 100)
-            h = max(8, pixbuf.get_height() * self.__scale / 100)
-            pixbuf = pixbuf.scale_simple(w, h, gtk.gdk.INTERP_BILINEAR)
-            image, mask = pixbuf.render_pixmap_and_mask(255)
+            surface_pixbuf = ninix.pix.create_pixbuf_from_file(path)
+            w = max(8, surface_pixbuf.get_width() * self.__scale / 100)
+            h = max(8, surface_pixbuf.get_height() * self.__scale / 100)
+            surface_pixbuf = surface_pixbuf.scale_simple(
+                w, h, gtk.gdk.INTERP_BILINEAR)
         except:
             self.callback['finalize']()
             return
-        self.w, self.h = image.get_size()
+        self.w, self.h = w, h
+        mask_pixmap = gtk.gdk.Pixmap(None, w, h, 1)
+        surface_pixbuf.render_threshold_alpha(
+            mask_pixmap, 0, 0, 0, 0, w, h, 1)
+        if self.window.is_composited():
+            self.window.input_shape_combine_mask(mask_pixmap, 0, 0)
+        else:
+            self.window.shape_combine_mask(mask_pixmap, 0, 0)
         self.darea.set_size_request(self.w, self.h)
-        self.darea.window.set_back_pixmap(image, False)        
-        self.window.shape_combine_mask(mask, 0, 0)
+        self.current_mask = mask_pixmap
+        self.current_surface_pixbuf = surface_pixbuf
+        self.darea.queue_draw()
 
     def set_position(self, reset=0):
         left, top, scrn_w, scrn_h = ninix.pix.get_workarea()
@@ -349,7 +369,7 @@ class Skin:
             self.x = max(left, self.x)
             self.y = min(top + scrn_h - self.h, self.y)
             self.y = max(top, self.y)
-        self.window.move(self.x, self.y)
+        self.window.move(self.x, self.y) ## FIXME
 
     def move(self, x_delta, y_delta): ## FIXME
         self.x = self.x + x_delta
@@ -396,6 +416,7 @@ class Skin:
     def leave_notify(self,  widget, event): ## FIXME
         pass
 
+
 class Balloon:
 
     def __init__(self):
@@ -404,6 +425,7 @@ class Balloon:
     def destroy(self): ## FIXME
         pass
 
+
 class Katochan:
 
     CATEGORY_LIST = ['pain',      # 痛い
@@ -462,8 +484,16 @@ class Katochan:
     def delete(self, widget, event):
         self.destroy()
 
-    def redraw(self, widget, event):
-        pass
+    def redraw(self, darea, event):
+        cr = darea.window.cairo_create()
+        cr.save()
+        cr.set_operator(cairo.OPERATOR_CLEAR)
+        cr.paint()
+        cr.restore()
+        cr.set_source_pixbuf(self.current_surface_pixbuf, 0, 0)
+        ##cr.paint_with_alpha(self.__alpha_channel)
+        cr.paint()
+        del cr
 
     def set_movement(self, timing):
         key = ''.join((timing, 'fall.type'))
@@ -506,24 +536,32 @@ class Katochan:
         self.x = target_x + target_w / 2 - self.w / 2 + \
                  self.offset_x * self.__scale / 100
         self.y = top + self.offset_y * self.__scale / 100
-        self.window.move(self.x, self.y)
+        self.window.move(self.x, self.y) ## FIXME
 
     def set_surface(self):
         path = os.path.join(self.data['dir'], 'surface%d.png' % self.id)
         try:
-            pixbuf = ninix.pix.create_pixbuf_from_file(path)
-            w = max(8, pixbuf.get_width() * self.__scale / 100)
-            h = max(8, pixbuf.get_height() * self.__scale / 100)
-            pixbuf = pixbuf.scale_simple(w, h, gtk.gdk.INTERP_BILINEAR)
-            image, mask = pixbuf.render_pixmap_and_mask(255)
+            surface_pixbuf = ninix.pix.create_pixbuf_from_file(path)
+            w = max(8, surface_pixbuf.get_width() * self.__scale / 100)
+            h = max(8, surface_pixbuf.get_height() * self.__scale / 100)
+            surface_pixbuf = surface_pixbuf.scale_simple(
+                w, h, gtk.gdk.INTERP_BILINEAR)
         except:
             self.callback['finalize']()
             return
-        self.w, self.h = image.get_size()
+        self.w, self.h = w, h
+        mask_pixmap = gtk.gdk.Pixmap(None, w, h, 1)
+        surface_pixbuf.render_threshold_alpha(
+            mask_pixmap, 0, 0, 0, 0, w, h, 1)
+        if self.window.is_composited():
+            self.window.input_shape_combine_mask(mask_pixmap, 0, 0)
+        else:
+            self.window.shape_combine_mask(mask_pixmap, 0, 0)
         self.darea.set_size_request(self.w, self.h)
-        self.darea.window.set_back_pixmap(image, False)
-        self.window.shape_combine_mask(mask, 0, 0)
-        self.darea.queue_draw_area(0, 0, self.w, self.h)
+        #self.darea.queue_draw_area(0, 0, self.w, self.h)
+        self.current_mask = mask_pixmap
+        self.current_surface_pixbuf = surface_pixbuf
+        self.darea.queue_draw()
 
     def load(self, data):
         self.data = data
@@ -573,18 +611,21 @@ class Katochan:
             offset_y = 32767
         self.offset_x = offset_x
         self.offset_y = offset_y
-        self.window = gtk.Window()
+        self.window = ninix.pix.TransparentWindow()
         self.window.set_title(self.data['name'])
         self.window.set_decorated(False)
         self.window.set_resizable(False)
         self.window.set_skip_taskbar_hint(True) # XXX
+        self.window.set_focus_on_map(False)
         self.window.connect('delete_event', self.delete)
         self.darea = gtk.DrawingArea()
         self.darea.set_events(gtk.gdk.EXPOSURE_MASK)
         self.darea.connect('expose_event', self.redraw)
-        self.darea.show()
+        self.darea.show() ## FIXME
         self.window.add(self.darea)
         self.darea.realize()
+        self.window.realize()
+        self.window.window.set_back_pixmap(None, False)
         self.window.show()
         self.id = 0
         self.set_surface()
