@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #  Copyright (C) 2001-2004 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2004-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2004-2011 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -25,6 +25,7 @@ import ninix.home
 if 'DISPLAY' in os.environ:
     import gtk
     import gobject
+    import cairo
     import ninix.pix
 
 # 注意:
@@ -234,6 +235,7 @@ class Catalog_xml:
             fileobj.write('</FileSet>\n')
         fileobj.write('</GhostList>\n')
 
+
 class Catalog(Catalog_xml):
 
     TYPE = ['Sakura', 'Kero']
@@ -254,6 +256,7 @@ class Catalog(Catalog_xml):
             except:
                 return ## FIXME
 
+
 #class InstallationManager(list):
 #
 #    def __init__(self, ngm):
@@ -266,6 +269,7 @@ class Catalog(Catalog_xml):
 #        a = self.master.ui.catalog.get(self.current_ghost, 'ArchiveUrl')
 #        os.system("%s '%s'" % (self.master.ninix_install_cmd, a))
 
+
 class SearchDialog:
 
     def __init__(self, callback):
@@ -312,6 +316,7 @@ class SearchDialog:
         self.hide()
         return True
 
+
 class UI:
     def __init__(self, callback):
         self.callback = callback
@@ -380,6 +385,7 @@ class UI:
         self.opened = 0
         self.textview = [None, None]
         self.darea = [None, None]
+        self.pixbuf = [None, None]
         self.info = None
         self.button = {}
         self.url = {}
@@ -488,9 +494,22 @@ class UI:
         darea = gtk.DrawingArea()
         vbox.pack_start(darea, False, True, 0)
         darea.show()
+        darea.set_events(gtk.gdk.EXPOSURE_MASK)
+        darea.connect('expose_event', self.redraw, side)
         self.darea[side] = darea
         return vbox
 
+    def redraw(self, darea, event, side):
+        cr = darea.window.cairo_create()
+        cr.save()
+        cr.set_operator(cairo.OPERATOR_CLEAR)
+        cr.paint()
+        cr.restore()
+        if self.pixbuf[side] is not None:
+            cr.set_source_pixbuf(self.pixbuf[side], 0, 0)
+            cr.paint()
+        del cr
+
     def update_surface_area(self):
         for side in [0, 1]:
             name = self.callback['get']('SakuraName') if side == 0 else \
@@ -503,16 +522,19 @@ class UI:
             if filename is not None:
                 try:
                     pixbuf = ninix.pix.create_pixbuf_from_file(filename)
-                    image, mask = pixbuf.render_pixmap_and_mask(255)
                 except:
-                    darea.window.set_back_pixmap(None, False)
+                    pixbuf = None
                 else:
-                    w, h = image.get_size()
+                    w = pixbuf.get_width()
+                    h = pixbuf.get_height()
                     darea.set_size_request(w, h)
-                    darea.window.set_back_pixmap(image, False)
+                    mask = gtk.gdk.Pixmap(None, w, h, 1)
+                    pixbuf.render_threshold_alpha(
+                        mask, 0, 0, 0, 0, w, h, 1)
                     darea.window.shape_combine_mask(mask, 0, 0)
-            else:                
-                darea.window.set_back_pixmap(None, False)
+            else:
+                pixbuf = None
+            self.pixbuf[side] = pixbuf
             darea.queue_draw()
 
     def create_info_area(self):
@@ -620,6 +642,7 @@ class UI:
         self.opened = 0
         return True
 
+
 class NGM:
 
     def __init__(self, callback):
