@@ -2,7 +2,7 @@
 #
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2011 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2003-2005 by Shun-ichi TAHARA <jado@flowernet.gr.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -18,6 +18,8 @@ import signal
 import sys
 import socket
 
+from ninix.home import search_plugins
+
 
 def send_script(sender, script, if_ghost=None):
     try:
@@ -44,13 +46,13 @@ def send_script(sender, script, if_ghost=None):
 
 class PluginControler:
 
-    def __init__(self, plugins, callback):
-        self.plugins = plugins
+    def __init__(self, callback):
         self.plugin_pids = []
         self.get_sstp_port = callback['get_sstp_port']
 
     def get_plugin_list(self):
         plugin_list = []
+        self.plugins = search_plugins()
         for i, plugin in enumerate(self.plugins):
             plugin_name = plugin[0]
             menu_items = plugin[3]
@@ -115,6 +117,7 @@ class PluginControler:
             os.setpgid(0, 0)
         except OSError:
             pass
-        for plugin_name, plugin_dir, startup, menu_items in self.plugins:
+        plugins = search_plugins()
+        for plugin_name, plugin_dir, startup, menu_items in plugins:
             if startup is not None:
                 self.exec_plugin(plugin_dir, startup)
