@@ -2,7 +2,7 @@
 #
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2011 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2003 by Shun-ichi TAHARA <jado@flowernet.gr.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -28,6 +28,11 @@ if 'DISPLAY' in os.environ:
     import gtk ## FIXME
     import glib
 
+try:
+    import gconf ## FIXME
+except:
+    gconf = None
+
 # For some reasons gst parses sys.argv on import. This is a workaround.
 # (See https://bugzilla.gnome.org/show_bug.cgi?id=549879 .)
 argv = sys.argv
@@ -45,6 +50,7 @@ import ninix.pix
 import ninix.script
 import ninix.version
 import ninix.update
+from ninix.home import get_ninix_home
 
 
 class Sakura:
@@ -129,9 +135,9 @@ class Sakura:
             'enqueue_event': self.enqueue_event,
             'getstring': self.getstring,
             'get_balloon_size': self.get_balloon_size,
-            'get_current_balloon_name': self.get_current_balloon_name,
+            'get_current_balloon_directory': self.get_current_balloon_directory,
+            'get_shell_list': self.get_shell_list,
             'get_keroname': self.get_keroname,
-            'get_own_balloon_name': self.get_own_balloon_name,
             'get_prefix': self.get_prefix,
             'get_selfname': self.get_selfname,
             'is_running': self.is_running,
@@ -144,6 +150,7 @@ class Sakura:
             'notify_surface_click': self.notify_surface_click,
             'notify_surface_mouse_motion': self.notify_surface_mouse_motion,
             'reset_idle_time': self.reset_idle_time,
+            'select_shell': self.select_shell,
             'select_balloon': self.select_balloon,
             'select_kinoko': self.select_kinoko,
             'select_nekodorif': self.select_nekodorif,
@@ -155,16 +162,21 @@ class Sakura:
             }
         for key in ['close_all', 'get_preference', 'edit_preferences',
                     'open_ghost_manager', 'show_usage', 'start_sakura',
-                    'get_ghost_list', 'get_shell_list', 'get_balloon_list',
+                    'get_ghost_list', 'get_balloon_list',
                     'get_nekodorif_list', 'get_kinoko_list',
                     'get_plugin_list', 'select_plugin']:
             surface_callback[key] = self.callback[key]
         self.surface = ninix.surface.Surface(surface_callback, debug)
         self.keep_silence(False)
-        desc, shiori_dir, use_makoto, surface_set, balloon, prefix, \
-            shiori_dll, shiori_name = data
-        self.new(desc, shiori_dir, use_makoto, surface_set, balloon, prefix,
-                 shiori_dll, shiori_name)
+        update_callback = {
+            'enqueue_event': self.enqueue_event,
+            'check_event_queue': self.check_event_queue,
+            }
+        self.updateman = ninix.update.NetworkUpdate(update_callback)
+        self.audio_player = gst.element_factory_make("playbin", "player")
+        fakesink = gst.element_factory_make("fakesink", "fakesink")
+        self.audio_player.set_property("video-sink", fakesink)
+        self.new(*data)
 
     def get_current_item(self):
         return self.current
@@ -195,27 +207,18 @@ class Sakura:
                 del self.__observer[i]
                 break
 
-    def new(self, desc, shiori_dir, use_makoto, surface_set, balloon, prefix,
+    def new(self, desc, shiori_dir, use_makoto, surface_set, prefix,
             shiori_dll, shiori_name): ## FIXME
         self.shiori = None
         self.desc = desc
         self.shiori_dir = shiori_dir
         self.use_makoto = use_makoto
         self.surface_set = surface_set
-        self.own_balloon = balloon
-        self.prefix = prefix
+        self.prefix = prefix ## FIXME: ninix_home should not be included.(append in get_prefix())
         self.shiori_dll = shiori_dll
         self.shiori_name = shiori_name
         name = (shiori_dll, shiori_name)
         self.shiori = self.__dll.request(name)
-        update_callback = {
-            'enqueue_event': self.enqueue_event,
-            'check_event_queue': self.check_event_queue,
-            }
-        self.updateman = ninix.update.NetworkUpdate(update_callback)
-        self.audio_player = gst.element_factory_make("playbin", "player")
-        fakesink = gst.element_factory_make("fakesink", "fakesink")
-        self.audio_player.set_property("video-sink", fakesink)
         char = 2
         while self.desc.get('char%d.seriko.defaultsurface' % char) is not None:
             char += 1
@@ -241,6 +244,17 @@ class Sakura:
             f.write('vanished_count, %s\n' % self.vanished_count)
             f.close()
 
+    def save_settings(self):
+        path = os.path.join(self.get_prefix(), 'SETTINGS')
+        try:
+            f = open(path, 'w')
+        except IOError, (code, message):
+            sys.stderr.write('cannot write %s\n' % path)
+        else:
+            if self.balloon_directory is not None:
+                f.write('balloon_directory, %s\n' % self.balloon_directory)
+            f.close()
+
     def load_history(self):
         path = os.path.join(self.get_prefix(), 'HISTORY')
         if os.path.exists(path):
@@ -272,6 +286,26 @@ class Sakura:
             self.ghost_time = 0
             self.vanished_count = 0
 
+    def load_settings(self):
+        path = os.path.join(self.get_prefix(), 'SETTINGS')
+        if os.path.exists(path):
+            balloon_directory = None
+            try:
+                f = open(path, 'r')
+            except IOError, (code, message):
+                sys.stderr.write('cannot read %s\n' % path)
+            else:
+                for line in f:
+                    if ',' not in line:
+                        continue
+                    key, value = [x.strip() for x in line.split(',', 1)]
+                    if key == 'balloon_directory':
+                        balloon_directory = value
+                f.close()
+            self.balloon_directory = balloon_directory
+        else:
+            self.balloon_directory = None
+
     def load_shiori(self):
         if self.shiori and self.shiori.load(self.shiori_dir):
             if getattr(self.shiori, 'show_description', None):
@@ -463,9 +497,6 @@ class Sakura:
         result = self.surface.get_position(side)
         return result if result is not None else (0, 0)
 
-#    def reset_balloon_fonts(self): ## FIXME: remove
-#        self.balloon.reset_fonts()
-
     def set_balloon_position(self, side, x, y):
         self.balloon.set_position(side, x, y)
 
@@ -656,7 +687,7 @@ class Sakura:
                 return
             self.notify_event('OnBoot', self.surface.name, default=default)
         else:
-            self.notify_event('OnShellChanged', name, name, path)
+            pass ## FIXME
 
     def notify_vanish_confirmation(self):
         def proc(self=self):
@@ -742,7 +773,9 @@ class Sakura:
     def notify_balloon_click(self, button, click, side):
         if self.script_mode == self.PAUSE_MODE:
             self.script_mode = self.BROWSE_MODE
-            self.balloon.redraw_arrow(self.script_side, 1)
+            self.balloon.clear_text_all()
+            self.balloon.hide_all()
+            self.script_side = 0
         elif button == 1 and click == 1:
             self.raise_all()
         if self.vanished:
@@ -760,7 +793,6 @@ class Sakura:
             if self.sstp_request_handler:
                 self.sstp_request_handler.send_sstp_break()
                 self.sstp_request_handler = None
-            self.notify_event('OnMouseDoubleClick', '', '', '', side, '')
         elif button == 3 and click == 1:
             if self.sstp_request_handler:
                 self.sstp_request_handler.send_sstp_break()
@@ -835,31 +867,64 @@ class Sakura:
     def get_prefix(self):
         return self.prefix
 
-    def get_own_balloon_name(self):
-        if self.own_balloon:
-            desc, balloon = self.own_balloon
-            name = desc.get('name', unicode(_('Own Balloon'), 'utf-8'))
-        else:
-            name = None
-        return name
+    def get_shell_list(self):
+        shell_list = []
+        set_type, i, j = self.current
+        for num in range(len(self.surface_set)):
+            name = self.surface_set[num][0]
+            value = (set_type, i, num)
+            working = 1 if num == j else 0 
+            shell_list.append((name, value, working))
+        return shell_list
 
-    def get_current_balloon_name(self):
-        return self.balloon.get_balloon_name()
+    def get_current_balloon_directory(self):
+        return self.balloon.get_balloon_directory()
+
+    def select_shell(self, widget, item):
+        set_type, i, j = item
+        assert self.current[0] == set_type
+        assert self.current[1] == i
+        assert self.current[2] != j
+        assert set_type == 'g'
+        assert self.surface_set and len(self.surface_set) > j
+        name, surface_dir, surface_desc, surface_alias, surface, surface_tooltips = \
+            self.surface_set[j]
+        if not name: ## FIXME
+            surface_name = unicode(_('Master'), 'utf-8')
+        else:
+            surface_name = name
+        def proc(self=self, item=item):
+            print 'ghost', item
+            self.current = item
+            self.set_surface(surface_desc, surface_alias, surface, surface_name,
+                             surface_dir, surface_tooltips)
+            self.surface.reset_alignment()
+            self.position_all()
+            self.notify_event('OnShellChanged',
+                              surface_name, surface_name, surface_dir)
+        self.enqueue_event('OnShellChanging', surface_name, surface_dir,
+                           proc=proc)
 
     def select_balloon(self, widget, item):
         if not widget.active:
             return
-        name = self.get_own_balloon_name()
-        if item != name:
-            desc, balloon = self.callback['get_balloon_description'](item)
-        else:
-            desc, balloon = self.own_balloon
+        if item == self.get_current_balloon_directory(): # no change
+            # need reloadning?
+            if self.balloon_directory != item:
+                self.balloon_directory = item # save user's choice
+            return
+        desc, balloon = self.callback['get_balloon_description'](item)
+        subdir = balloon['balloon_dir'][0]
+        path = os.path.join(get_ninix_home(), 'balloon', subdir)
         self.balloon.hide_all()
         self.set_balloon(desc, balloon)
+        if item == subdir:
+            self.balloon_directory = subdir # save user's choice
         self.balloon.set_balloon_default()
-        print 'balloon', item
-        path = '' ## FIXME
-        self.notify_event('OnBalloonChange', item, path)
+        self.position_balloons()
+        name = desc.get('name', '')
+        print 'balloon', name, path
+        self.notify_event('OnBalloonChange', name, path)
 
     def surface_bootup(self):
         for side in [0, 1]:
@@ -1009,26 +1074,13 @@ class Sakura:
         self.__temp_mode = temp
         self.current = item
         set_type, i, j = item
-        if set_type == 'g':
-            print 'ghost', item
-            if not self.surface_set:
-                shell_name, surface_set, balloon = self.callback['request_shell'](0)
-                name, surface_dir, surface_desc, surface_alias, surface, surface_tooltips = \
-                      surface_set[0]
-                surface_set = None
-            else:
-                name, surface_dir, surface_desc, surface_alias, surface, surface_tooltips = \
-                      self.surface_set[j]
-                surface_set = self.surface_set
-                balloon = self.own_balloon
-        elif set_type == 's':
-            print 'shell', item
-            shell_name, surface_set, balloon = self.callback['request_shell'](i)
-            name, surface_dir, surface_desc, surface_alias, surface, surface_tooltips = \
-                  surface_set[j]
-        if not surface_set:
-            surface_name = None
-        elif len(surface_set) == 1:
+        assert set_type == 'g'
+        print 'ghost', item
+        assert self.surface_set and len(self.surface_set) > j
+        name, surface_dir, surface_desc, surface_alias, surface, surface_tooltips = \
+            self.surface_set[j]
+        surface_set = self.surface_set
+        if not name: ## FIXME
             surface_name = unicode(_('Master'), 'utf-8')
         else:
             surface_name = name
@@ -1036,15 +1088,23 @@ class Sakura:
             name = prev_name
         self.set_surface(surface_desc, surface_alias, surface, surface_name,
                          surface_dir, surface_tooltips)
-        if self.callback['get_preference']('ignore_default') or not balloon:
+        self.load_settings()
+        if self.callback['get_preference']('ignore_default'): ## FIXME: change prefs key
+            default_balloon = self.callback['get_preference']('default_balloon')
+            balloon = self.callback['get_balloon_description'](default_balloon)
+        elif self.balloon_directory is not None:
+            balloon = self.callback['get_balloon_description'](self.balloon_directory)
+        else:
+            balloon_path = self.desc.get('deault.balloon.path', '')
             balloon_name = self.desc.get('balloon', '')
-            number = None
-            if balloon_name:
-                number = self.callback['find_balloon_by_name'](balloon_name)
-            if number is None:
+            balloon = None
+            if balloon_path:
+                balloon = self.callback['find_balloon_by_subdir'](balloon_path)
+            if balloon is None and balloon_name:
+                balloon = self.callback['find_balloon_by_name'](balloon_name)
+            if balloon is None:
                 default_balloon = self.callback['get_preference']('default_balloon')
-                number = self.callback['find_balloon_by_name'](default_balloon) or 0
-            balloon = self.callback['request_balloon'](number)
+                balloon = self.callback['get_balloon_description'](default_balloon)
         desc, balloon = balloon
         self.set_balloon(desc, balloon)
         if not temp:
@@ -1071,6 +1131,7 @@ class Sakura:
             return
         self.notify_observer('finalize')
         self.__running = 0
+        self.save_settings()
         self.save_history()
         self.callback['rebuild_ghostdb'](self, None)
         self.hide_all()
@@ -1100,7 +1161,6 @@ class Sakura:
                 keyword = {'event_type': 'NOTIFY'}
                 self.notify_event(*args, **keyword)
             self.old_otherghostname = otherghostname
-        mikire, kasanari = self.surface.get_mikire_kasanari()
         if not self.__running:
             pass
         elif self.script_mode == self.PAUSE_MODE:
@@ -1154,12 +1214,14 @@ class Sakura:
                 self.keep_silence(False)
         elif self.clock[0] != second and \
              self.notify_event('OnSecondChange', self.get_uptime(),
-                               mikire, kasanari,
+                               self.surface.get_mikire(),
+                               self.surface.get_kasanari(),
                                not self.passivemode and self.cantalk):
             pass
         elif self.clock[1] != minute and \
              self.notify_event('OnMinuteChange', self.get_uptime(),
-                               mikire, kasanari,
+                               self.surface.get_mikire(),
+                               self.surface.get_kasanari(),
                                not self.passivemode and self.cantalk):
             pass
         elif self.surface_mouse_motion is not None:
@@ -1202,6 +1264,7 @@ class Sakura:
             sys.stdout.write('reloading....\n')
             self.shiori.unload()
             self.callback['stop_sakura'](self, self.callback['reload_current_sakura'], self)
+            self.load_settings()
             self.restart() ## FIXME
             sys.stdout.write('done.\n')
             self.enqueue_event(*self.reload_event)
@@ -1279,7 +1342,7 @@ class Sakura:
         if chr_id >= 0:
             self.script_side = chr_id
 
-    def __yen_4(self, args): ## FIXME
+    def __yen_4(self, args):
         if self.script_side == 0:
             sw, sh = self.get_surface_size(1)
             sx, sy = self.get_surface_position(1)
@@ -1292,21 +1355,15 @@ class Sakura:
         x, y = self.get_surface_position(self.script_side)
         left, top, scrn_w, scrn_h = ninix.pix.get_workarea()
         if sx + sw / 2 > left + scrn_w / 2:
-            new_x = sx - w - int(scrn_w / 20)
+            new_x = min(x - scrn_w // 20, sx - scrn_w // 20)
         else:
-            new_x = sx + sw + int(scrn_w / 20)
-        new_x = max(new_x, left)
-        new_x = min(new_x, left + scrn_w - w)
+            new_x = max(x + scrn_w // 20, sx + scrn_w // 20)
         step = -10 if x > new_x else 10
-        if abs(sx - new_x) < abs(sx - x):
-            return
         for current_x in range(x, new_x, step):
             self.set_surface_position(self.script_side, current_x, y)
-            self.balloon.reset_balloon_all()
         self.set_surface_position(self.script_side, new_x, y)
-        self.balloon.reset_balloon_all()
 
-    def __yen_5(self, args): ## FIXME
+    def __yen_5(self, args):
         if self.script_side == 0:
             sw, sh = self.get_surface_size(1)
             sx, sy = self.get_surface_position(1)
@@ -1329,9 +1386,7 @@ class Sakura:
         step = -10 if x > new_x else 10
         for current_x in range(x, new_x, step):
             self.set_surface_position(self.script_side, current_x, y)
-            self.balloon.reset_balloon_all()
         self.set_surface_position(self.script_side, new_x, y)
-        self.balloon.reset_balloon_all()
 
     def __yen_s(self, args):
         surface_id = args[0]
@@ -1455,8 +1510,6 @@ class Sakura:
     def __yen_x(self, args):
         if self.script_mode == self.BROWSE_MODE:
             self.script_mode = self.PAUSE_MODE
-            self.balloon.redraw_arrow(self.script_side, 1)
-            self.balloon.append_text(self.script_side, u'\n')
 
     def __yen_a(self, args):
         self.start_script(self.getaistringrandom())
@@ -1553,7 +1606,13 @@ class Sakura:
         elif args[0:2] == ['open', 'configurationdialog']:
             self.callback['edit_preferences']()
         elif args[0:2] == ['change', 'shell'] and argc > 2:
-            self.callback['select_shell_by_name'](self, args[2], 0)
+            set_type, i, j = self.current
+            for num in range(len(self.surface_set)):
+                shell_name = self.surface_set[num][0]
+                if shell_name == args[2]:
+                    item = (set_type, i, num)
+                    self.select_shell(None, item) ## FIXME
+                    break
         elif args[0:2] == ['change', 'ghost'] and argc > 2:
             if args[2] == 'random':
                 self.callback['select_ghost'](self, 0, 0)
@@ -1604,6 +1663,32 @@ class Sakura:
                 self.surface.window_stayontop(True)
             elif args[2] == '!stayontop':
                 self.surface.window_stayontop(False)
+        elif args[0:2] == ['set', 'wallpaper']:
+            path = os.path.join(self.get_prefix(), args[2])
+            opt = None
+            if len(args) > 3:
+                # possible picture_options value:
+                # "none", "wallpaper", "centered", "scaled", "stretched"
+                options = {
+                    'center': 'centered',
+                    'tile': 'wallpaper',
+                    'stretch': 'stretched'
+                    }
+                if args[3] not in options:
+                    opt = None
+                else:
+                    opt = options[args[3]]
+            if opt is None:
+                opt = 'centered' # default
+            if os.path.exists(path) and gconf is not None:
+                client = gconf.client_get_default() 
+                gconf_background_dir = '/desktop/gnome/background/'
+                client.set_string(''.join((gconf_background_dir,
+                                           'picture_filename')),
+                                   path) 
+                client.set_string(''.join((gconf_background_dir,
+                                           'picture_options')),
+                                   opt) 
         elif args[0] == '*':
             self.balloon.append_sstp_marker(self.script_side)
         else:
