@@ -2,7 +2,7 @@
 #
 #  Copyright (C) 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2011 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -24,16 +24,18 @@ def print_error(message):
 
 class Controler:
 
+    DEFAULT_FPS = 30.0 # current default
+
     def __init__(self, seriko):
         self.seriko = seriko
         self.exclusive_actor = None
         self.base_id = None
         self.timeout_id = None
-        self.idle_id = None
         self.reset_overlays()
         self.queue = []
-        self.frame = {'step': 0.033, # [sec]
-                      'next': 0}
+        self.fps = self.DEFAULT_FPS
+        self.next_tick = 0
+        self.prev_tick = 0 # XXX
         self.active = []
         self.__move = None
         self.__dirty = 1
@@ -43,9 +45,11 @@ class Controler:
         return not bool(self.__inactive)
 
     def set_base_id(self, window, surface_id):
-        if surface_id == -2:
+        if surface_id == '-2':
             self.terminate(window)
             self.base_id = window.surface_id
+        elif surface_id == '-1':
+            self.base_id = window.surface_id
         else:
             self.base_id = surface_id
         self.__dirty = 1
@@ -61,61 +65,58 @@ class Controler:
             glib.source_remove(self.timeout_id)
         if quality is None:
             quality = 1.0
-        step = 0.033 / quality ## FIXME
-        self.frame['step'] = step
+        self.fps = self.DEFAULT_FPS * quality
         if self.timeout_id is not None:
-            self.timeout_id = glib.timeout_add(int(step * 1000.0), self.update, window)
+            self.timeout_id = glib.timeout_add(
+                int(1000.0 / self.fps), # [msec]
+                self.update, window)
 
     def set_inactive(self, flag, window):
         self.__inactive = bool(flag)
-        self.reset(window, window.surface_id)
+        self.__dirty = 1 # XXX
 
     def update_frame(self, window):
-        if self.terminate_flag:
-            return False
-        if len(window.frame_buffer) > 1.0 / self.frame['step']: # XXX
-            return True
-        self.frame['next'] += self.frame['step']
         frame, actor = self.get_actor_next(window)
         last_actor = actor
         while actor is not None:
             actor.update(window, frame)
             last_actor = actor
             frame, actor = self.get_actor_next(window)
-        self.update_surface_frame_buffer(window)
         if last_actor is not None and last_actor.exclusive and \
            last_actor.terminate_flag and self.exclusive_actor is None: # XXX
             self.invoke_restart(window)
-        self.idle_id = glib.timeout_add(10, self.update_frame, window) # XXX
-        return False
 
     def get_actor_next(self, window):
         if self.active:
             self.active.sort()
-            if self.active[0][0] <= self.frame['next']:
+            if self.active[0][0] <= self.next_tick:
                 return self.active.pop(0)
         return None, None
 
-    def update_surface_frame_buffer(self, window):
-        window.update_frame_buffer(
-            self, self.frame['next'], self.__move, self.__dirty)
-        self.__dirty = 0
-
     def update(self, window):
-        if self.terminate_flag:
-            return False
-        if not self.__inactive:
-            window.update()
-        self.timeout_id = glib.timeout_add(int(self.frame['step'] * 1000), self.update, window)
-        return False
+        ## FIXME: Use g_source_get_time. (glib.glib_version > (2, 27, 3))
+        current_tick = glib.get_current_time() # [sec]
+        if self.prev_tick == 0: ## First time
+            delta_tick = 1000.0 / self.fps # [msec]
+        else:
+            delta_tick = (current_tick - self.prev_tick) * 1000.0 # [msec]
+        self.next_tick += delta_tick
+        self.prev_tick = current_tick
+        self.update_frame(window)
+        if self.__dirty:
+            window.update_frame_buffer()
+            self.__dirty = 0
+        if self.__move is not None:
+            window.move_surface(*self.__move)
+            self.__move = None
+        return True
 
     def lock_exclusive(self, window, actor):
         assert self.exclusive_actor is None
         self.terminate(window)
         self.exclusive_actor = actor
         actor.set_post_proc(self.unlock_exclusive, (window, actor))
-        self.update_frame(window) # XXX
-        self.__dirty = 1
+        self.__dirty = 1 # XXX
 
     def unlock_exclusive(self, window, actor):
         assert self.exclusive_actor == actor
@@ -150,7 +151,7 @@ class Controler:
             return
         if actor.exclusive:
             self.lock_exclusive(window, actor)
-        actor.invoke(window, self.frame['next'])
+        actor.invoke(window, self.next_tick)
 
     def invoke(self, window, actor_id, update=0):
         if self.base_id not in self.seriko:
@@ -218,34 +219,23 @@ class Controler:
     def reset(self, window, surface_id):
         self.queue = []
         self.terminate(window)
-        self.frame['next'] = 0
+        self.next_tick = 0
+        self.prev_tick = 0 # XXX
         self.set_base_id(window, window.surface_id)
         self.base_id = surface_id if surface_id in self.seriko else \
             window.surface_id
-        if self.base_id in self.seriko:
-            surface_ids = []
-            for actor in self.seriko[self.base_id]:
-                for surface_id in actor.get_surface_ids():
-                    if surface_id not in surface_ids:
-                        surface_ids.append(surface_id)
-            if surface_ids:
-                window.prefetch(surface_ids)
-        self.update_frame(window)
+        self.__dirty = 1 # XXX
 
     def start(self, window):
         self.invoke_runonce(window)
         self.invoke_always(window)
-        self.timeout_id = glib.timeout_add(int(self.frame['step'] * 1000),
-                                           self.update, window)
-
-    def terminate(self, window):
-        self.terminate_flag = 1
         if self.timeout_id is not None:
             glib.source_remove(self.timeout_id)
-            self.timeout_id = None
-        if self.idle_id is not None:
-            glib.source_remove(self.idle_id)
-            self.idle_id = None
+        self.timeout_id = glib.timeout_add(
+            int(1000.0 / self.fps), # [msec]
+            self.update, window)
+
+    def terminate(self, window):
         if self.base_id in self.seriko:
             for actor in self.seriko[self.base_id]:
                 actor.terminate()
@@ -253,8 +243,11 @@ class Controler:
         self.active = []
         self.__move = None
         self.__dirty = 1
-        window.clear_frame_buffer()
-        self.terminate_flag = 0
+
+    def destroy(self):
+        if self.timeout_id is not None:
+            glib.source_remove(self.timeout_id)
+            self.timeout_id = None
 
     def iter_overlays(self):
         actors = self.overlays.keys()
@@ -266,6 +259,7 @@ class Controler:
             ##print 'actor=%d, id=%s, x=%d, y=%d' % (actor.get_id(), pixbuf_id, x, y)
             yield pixbuf_id, x, y
 
+
 class Actor:
 
     def __init__(self, actor_id, interval):
@@ -335,6 +329,7 @@ class Actor:
             raise RuntimeError, 'should not reach here'
         self.last_method = method
 
+
 class ActiveActor(Actor): # always
 
     def __init__(self, actor_id, interval):
@@ -358,9 +353,10 @@ class ActiveActor(Actor): # always
         if self.pattern == len(self.patterns):
             self.pattern = 0
         self.show_pattern(window, surface, method, args)
-        window.append_actor(base_frame + interval / 1000.0, self)
+        window.append_actor(base_frame + interval, self)
         return False
 
+
 class RandomActor(Actor): # sometimes, rarely, randome
 
     def __init__(self, actor_id, interval, wait_min, wait_max):
@@ -377,7 +373,7 @@ class RandomActor(Actor): # sometimes, rarely, randome
         self.terminate()
         self.terminate_flag = 0
         self.reset()
-        window.append_actor(base_frame + self.wait / 1000.0, self)
+        window.append_actor(base_frame + self.wait, self)
 
     def update(self, window, base_frame):
         if self.terminate_flag:
@@ -391,9 +387,10 @@ class RandomActor(Actor): # sometimes, rarely, randome
         else:
             self.reset()
         self.show_pattern(window, surface, method, args)
-        window.append_actor(base_frame + self.wait / 1000.0, self)
+        window.append_actor(base_frame + self.wait, self)
         return False
 
+
 class OneTimeActor(Actor): # runone
 
     def __init__(self, actor_id, interval):
@@ -422,9 +419,10 @@ class OneTimeActor(Actor): # runone
             self.terminate()
         self.show_pattern(window, surface, method, args)
         if self.wait >= 0:
-            window.append_actor(base_frame + self.wait / 1000.0, self)
+            window.append_actor(base_frame + self.wait, self)
         return False
 
+
 class PassiveActor(Actor): # never, yen-e, talk
 
     def __init__(self, actor_id, interval):
@@ -452,9 +450,10 @@ class PassiveActor(Actor): # never, yen-e, talk
             self.terminate()
         self.show_pattern(window, surface, method, args)
         if self.wait >= 0:
-            window.append_actor(base_frame + self.wait / 1000.0, self)
+            window.append_actor(base_frame + self.wait, self)
         return False
 
+
 class Mayuna(Actor):
 
     def set_exclusive(self):
@@ -690,5 +689,6 @@ def test():
             for pattern in actor.get_patterns():
                 print 'surface=%s, interval=%d, method=%s, args=%s' % pattern
 
+
 if __name__ == '__main__':
     test()
