@@ -2,7 +2,7 @@
 #
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2011 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -13,6 +13,7 @@
 #
 
 import urlparse
+import urllib
 import StringIO
 import os
 import hashlib
@@ -121,7 +122,9 @@ class NetworkUpdate:
             self.end_updates()
         elif (self.state - 5) % 5 == 0:
             filename, checksum = self.schedule[0]
-            self.download(os.path.join(self.path, filename), event=1)
+            print 'UPDATE:', filename, checksum
+            self.download(os.path.join(self.path, urllib.quote(filename)),
+                          event=1)
         elif (self.state - 5) % 5 == 1:
             self.connect()
         elif (self.state - 5) % 5 == 2:
@@ -130,7 +133,8 @@ class NetworkUpdate:
             self.get_content()
         elif (self.state - 5) % 5 == 4:
             filename, checksum = self.schedule.pop(0)
-            self.update_file(filename, checksum)
+            self.update_file(unicode(filename, 'Shift_JIS').encode('utf-8'),
+                             checksum)
         return 1
 
     def start_updates(self):
@@ -289,7 +293,8 @@ class NetworkUpdate:
                 return None
             if not filename:
                 continue
-            path = os.path.join(self.ghostdir, self.adjust_path(filename))
+            path = os.path.join(self.ghostdir, self.adjust_path(
+                    unicode(filename, 'Shift_JIS').encode('utf-8')))
             try:
                 data = open(path).read()
             except IOError:
@@ -386,7 +391,7 @@ class NetworkUpdate:
             line = line.strip()
             if not line:
                 continue
-            filename = line
+            filename = unicode(line, 'Shift_JIS').encode('utf-8')
             filelist.append(filename.replace('\\', '/').lower())
         if error is not None:
             print error
