@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-#  Copyright (C) 2005-2010 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2005-2011 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -10,8 +10,8 @@
 #  PURPOSE.  See the GNU General Public License for more details.
 #
 
-NUMBER = '4.0.8'
-CODENAME = 'jump off into never-never land'
+NUMBER = '4.1.9'
+CODENAME = 'black magic'
 
 VERSION = '%s (%s)' % (NUMBER, CODENAME)
 VERSION_INFO = (''.join((r'\h\s[0]\w8ninix-aya %s\n' % VERSION,
@@ -21,6 +21,6 @@ VERSION_INFO = (''.join((r'\h\s[0]\w8ninix-aya %s\n' % VERSION,
                          r'\_q'
                          r'Copyright (c) 2001, 2002 Tamito KAJIYAMA\n'
                          r'Copyright (c) 2002-2006 MATSUMURA Namihiko\n'
-                         r'Copyright (c) 2002-2010 Shyouzou Sugitani\n'
+                         r'Copyright (c) 2002-2011 Shyouzou Sugitani\n'
                          r'Copyright (c) 2002, 2003 ABE Hideaki\n'
                          r'Copyright (c) 2003-2005 Shun-ichi TAHARA\e')))
